/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	Main file for the ArrayList XFCN. See manual for more details
	on program, its use, and its organization.
*/

#include <MyHeaders>
#include "ListMain.h"
#include "strlib.h"
#include "codelib.h"
#include "lookuptabl.h"

/*--------------------------------------------------------------------*/
/* Prototypes
/*--------------------------------------------------------------------*/

static Boolean	Initialize(XCmdBlockPtr param, Handle rsrc,
									Boolean has128k, GlobalData *gdataPtr);
static void Execute(GlobalData gdata);
static OSErr DoExecute(GlobalData gdata, short arg, CommandType command);

static OSErr param2command(GlobalData gdata, short arg, CommandType *command);

/*--------------------------------------------------------------------*/
/* The main routine
/*--------------------------------------------------------------------*/

int codersrc_main(XCmdBlockPtr param, Handle rsrc, Boolean has128k)
{	
	GlobalData	gdata; /* global data used by the program */

	if (Initialize(param, rsrc, has128k, &gdata))
		Execute(gdata);
	return(0);
}

static Boolean Initialize(XCmdBlockPtr param,
								Handle rsrc,
								Boolean has128k,
								GlobalData *gdataPtr)
{
	if (! has128k) {
		result2num(param, ERR_BASIC_ROMVER);
		return(false);
	}
	result2handle(param, NULL);
	if (InitGlobalData(param, rsrc, gdataPtr) != noErr)
		return(false);
	return(true);
}

/*--------------------------------------------------------------------*/
/* Functions for executing the specified command
/*--------------------------------------------------------------------*/

static void Execute(GlobalData gdata)
{
	StrListType	strlist;	/* strlist to operate on */
	CommandType	command;	/* command to execute */
	OSErr			err = noErr;

	/* get and then execute the command */
	err = param2command(gdata, 0, &command);
	if (! err)
		err = DoExecute(gdata, 1, command);
	
	/* remember result code */
	if (command != COMMAND_ERROR)
		SetError(gdata, err);

	/* The following commands set the return value to the error code if
		there was an error. This makes it simpler to check for errors
		from the script since the script doesn't need to make an
		extra call with the Error command to find out if there was
		an error.
	*/
	if (err) {
		switch (command) {
		case COMMAND_NULL:
		case COMMAND_ADD:
		case COMMAND_NEW:
		case COMMAND_DISPOSE:
		case COMMAND_SORT:
		case COMMAND_QKSORT:
		case COMMAND_SHSORT:
		case COMMAND_INSERT:
		case COMMAND_DELETE:
		case COMMAND_PUSH:
		case COMMAND_SET:
		case COMMAND_SETATTR:
		case COMMAND_SETDIM:
		case COMMAND_UNION:
		case COMMAND_DIFFERENCE:
		case COMMAND_INTERSECTION:
			result2error(GetParam(gdata), err);
			break;
		}
	}
}

/* execute the specified command */
static OSErr DoExecute(GlobalData gdata, short arg, CommandType command)
{
	ArrayType array = NULL;
	ArraySub sub;
	OSErr err = noErr;
	
	/* get the strlist if needed for this command */
	switch (command) {
	case COMMAND_ERROR:
	case COMMAND_VERSION:
	case COMMAND_USAGE:
	case COMMAND_NEW:
	case COMMAND_DISPOSE:
	case COMMAND_UNION:
	case COMMAND_DIFFERENCE:
	case COMMAND_INTERSECTION:
		break; /* do nothing */
	default:
		err = param2array(gdata, arg++, &array, &sub);
		if (err) return(err);
		break;
	}
	
	/* execute the command */
	switch (command) {
	case COMMAND_VERSION:	return(ExecVersion(gdata, arg));
	case COMMAND_USAGE:		return(ExecUsage(gdata, arg));
	case COMMAND_ERROR:		return(ExecError(gdata, arg));
	case COMMAND_NEW:			return(ExecNew(gdata, arg, true));
	case COMMAND_DISPOSE:	return(ExecDispose(gdata, arg));
	case COMMAND_INSERT:		return(ExecInsert(gdata, arg, array, &sub));
	case COMMAND_DELETE:		return(ExecDelete(gdata, arg, array, &sub));
	case COMMAND_SET:			return(ExecSet(gdata, arg, array, &sub));
	case COMMAND_GET:			return(ExecGet(gdata, arg, array, &sub));
	case COMMAND_ADD:			return(ExecAdd(gdata, arg, array, &sub));
	case COMMAND_GETATTR:	return(ExecGetAttr(gdata, arg, array, &sub));
	case COMMAND_SETATTR:	return(ExecSetAttr(gdata, arg, array, &sub));
	case COMMAND_GETDIM:		return(ExecGetDim(gdata, arg, array, &sub));
	case COMMAND_SETDIM:		return(ExecSetDim(gdata, arg, array, &sub));
	case COMMAND_SIZE:		return(ExecSize(gdata, arg, array, &sub));
	case COMMAND_SEARCH:		return(ExecSearch(gdata, arg, array, &sub));
	case COMMAND_SEQSEARCH:	return(ExecSeqSearch(gdata, arg, array, &sub));
	case COMMAND_BINSEARCH:	return(ExecBinSearch(gdata, arg, array, &sub));
	case COMMAND_SORT:		return(ExecSort(gdata, arg, array, &sub));
	case COMMAND_SHSORT:		return(ExecShSort(gdata, arg, array, &sub));
	case COMMAND_QKSORT:		return(ExecQkSort(gdata, arg, array, &sub));
	case COMMAND_PUSH:		return(ExecPush(gdata, arg, array, &sub));
	case COMMAND_POP:			return(ExecPop(gdata, arg, array, &sub));
	case COMMAND_TOP:			return(ExecTop(gdata, arg, array, &sub));
	case COMMAND_UNION:		return(ExecUnion(gdata, arg));
	case COMMAND_DIFFERENCE:return(ExecDiff(gdata, arg));
	case COMMAND_INTERSECTION:return(ExecInter(gdata, arg));
	}
	return(ERR_BASIC_COMMAND);
}

/*--------------------------------------------------------------------*/
/* Parameter extraction functions.
/*--------------------------------------------------------------------*/

/* convert parameter into an array and subscript */
OSErr param2array(GlobalData gdata, short arg, ArrayType *arrayp, ArraySub *sub)
{
	String	name;			/* name of array */
	CStr255	subscript;	/* subscripts */
	ArraySub	dim;			/* dimensions of array */
	OSErr		err;
	
	if ((err=param2arrayname(gdata, arg, &name)) == noErr) {
		/* lookup array in table */
		if ((err=ltable_get(GetTable(gdata), name, arrayp)) == noErr) {
			/* get array's dimensions and parse subscripts */
			if ((err = param2subscript(gdata, arg, subscript)) == noErr) {
				if ((err = array_getdim(*arrayp, &dim)) == noErr)
					if (! parse_subscript(subscript, &dim, sub))
						err = ERR_ARRAY_SUBSCRIPT;
			}
		}
		/* dispose of name of array */
		strfree(name);
	}
	return(err);
}

/* convert parameter into an array name; returns copy of name */
OSErr param2arrayname(GlobalData gdata, short arg, String *name)
{
	CStr255	str;
	size_t	len;
	OSErr		err;
	
	*name = NULL;
	if ((err=param2handle(GetParam(gdata), arg++, name)) == noErr) {
		/* copy name and terminate it at first subscript start char */
		strncpy(str, **name, sizeof(CStr255)-1); str[sizeof(CStr255)-1] = 0;
		len = substrlen(str, SUBSCRIPT_START);
		str[len] = 0;
		/* allocate name */
		*name = stralloc(str);
		err = MemError();
	}
	return(err);
}

/* extract the subscript part of the parameter */
OSErr param2subscript(GlobalData gdata, short arg, CStr255 subscript)
{
	String name;
	size_t len;
	OSErr	err;
	
	*subscript = 0;
	if ((err=param2handle(GetParam(gdata), arg++, &name)) == noErr) {
		/* skip to first substring start char */
		len = substrlen(*name, SUBSCRIPT_START);
		/* copy remainder of string */
		strncpy(subscript, *name+len, sizeof(CStr255)-1);
		subscript[sizeof(CStr255)-1] = 0;
	}
	return(err);
}

/* translate command name into a command number */
static OSErr param2command(GlobalData gdata, short arg, CommandType *command)
{
	String	str;
	OSErr		err;
	short		index;

	*command = COMMAND_NULL;
	if ((err = param2handle(GetParam(gdata), arg++, &str)) == noErr)
		err = hlookupTabl(GetRsrcID(gdata, RSRC_COMMANDS_ID), str, &index);
	if (err) return(err);
	*command = index;
	if (index <= COMMAND_NULL || COMMAND_LAST <= index)
		return(ERR_BASIC_COMMAND);
	return(noErr);
}

/* translate attribute name into an attribute number */
OSErr	param2attr(GlobalData gdata, short arg, StrListAttr *attr)
{
	String	str;
	OSErr		err;
	short		index;

	if ((err = param2handle(GetParam(gdata), arg++, &str)) == noErr)
		err = hlookupTabl(GetRsrcID(gdata, RSRC_ATTRIBUTES_ID), str, &index);
	if (err) return(err);
	*attr = index;
	if (index <= AttrNULL || AttrLast <= index)
		return(ERR_STRLIST_ATTR);
	return(noErr);
}

/* translate a compare attribute name into a number */
OSErr param2cmpr(GlobalData gdata, short arg, StrCompareType *cmpr)
{
	String	str;
	OSErr		err;
	short		index;

	if ((err = param2handle(GetParam(gdata), arg++, &str)) == noErr)
		err = hlookupTabl(GetRsrcID(gdata, RSRC_COMPARES_ID), str, &index);
	if (err) return(err);
	*cmpr = index;
	switch (index) {
	case STR_COMPARE_EXACT:
	case STR_COMPARE_IGNORECASE:
	case STR_COMPARE_INTERNATIONAL:
	case STR_COMPARE_FLOAT:
		return(noErr);
	}
	return(ERR_STRLIST_ATTR);
}

/* translate a comparison attribute number into a string */
void result2cmpr(GlobalData gdata, StrCompareType cmpr)
{
	CStr255	str;
	OSErr		err;
	
	err = lookupTablNum(GetRsrcID(gdata, RSRC_COMPARES_ID), cmpr, str);
	if (! err)
		result2string(GetParam(gdata), str);
}
