/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	Main header for ArrayList XFCN. See manual for more details.
*/

#ifndef _H_ListMain
#define _H_ListMain

#ifndef _H_hyperlib
#include "hyperlib.h"
#endif

#ifndef _H_listtable
#include "listtable.h"
#endif

/*------------------------------------------------------------------------*/
/* resources */
/*------------------------------------------------------------------------*/

/* Resource IDs are stored internally as unique integers.
	The IDs are passed to a function that maps these internal
	numbers to the IDs of the actual resources in the resource file.
	The map is initialized the first time we are called.
*/

/* internal resource IDs */
enum {
	RSRC_INFO_ID,			/* info strings */
	RSRC_COMMANDS_ID,		/* command words */
	RSRC_ATTRIBUTES_ID,	/* attribute words */
	RSRC_COMPARES_ID,		/* string comparison methods */ 
	RSRC_COUNT				/* number of resources */
};

/* the info strings */
enum {
	RSRC_VERSION_INDEX=1,/* version info */
	RSRC_USAGE_INDEX		/* usage info */
};

/*------------------------------------------------------------------------*/
/* numbers of the command strings */
/*------------------------------------------------------------------------*/

typedef enum {
	COMMAND_NULL,
	COMMAND_VERSION,
	COMMAND_USAGE,
	COMMAND_ERROR,
	
	COMMAND_NEW,
	COMMAND_DISPOSE,

	COMMAND_GETDIM,
	COMMAND_SETDIM,
	COMMAND_SIZE,

	COMMAND_INSERT,
	COMMAND_DELETE,
	COMMAND_ADD,
	
	COMMAND_GET,
	COMMAND_SET,

	COMMAND_SEARCH,
	COMMAND_SEQSEARCH,
	COMMAND_BINSEARCH,
	
	COMMAND_QKSORT,
	COMMAND_SHSORT,
	COMMAND_SORT,

	COMMAND_PUSH,
	COMMAND_POP,
	COMMAND_TOP,

	COMMAND_UNION,
	COMMAND_DIFFERENCE,
	COMMAND_INTERSECTION,

	COMMAND_GETATTR,
	COMMAND_SETATTR,

	COMMAND_LAST
} CommandType;

/*------------------------------------------------------------------------*/
/* GlobalData */
/*------------------------------------------------------------------------*/

#define CHKSUM	'ALST'

/*	Structure containing various information that must be stored
	between invocations of the code resource. A handle to this
	data is stored in a HyperCard global variable. The first
	field (chksum) is set to a unique value, which aids in detecting
	a faulty handle or a handle belonging to an incompatible version
	of ArrayList. The table field contains a list of the names of
	all of the lists created by ArrayList. (Corresponding to each
	item in the names list is a handle to a string list.)
	The baseID field contains the base reference ID for
	the code resource and all resources it uses. This ID is the same
	as the ID of the code resource itself. The error field contains
	the error that occured on the last call to this code resource.
	The param field contains a pointer to the parameter block
	passed from HyperCard.
	
	Most functions only make used of the param field, but all functions
	should have access to all of the fields.
*/ 
typedef struct {
	long				chksum;	/* checksum to guard against corrupted data */
	XCmdBlockPtr	param;	/* HyperCard's parameter block */
	ListTableType	table;	/* table of arrays */
	ResID				baseID;	/* ID of code resource */
	OSErr				error;	/* last error */
	ResID				map[RSRC_COUNT]; /* resource map */
} GlobalDataStruct, **GlobalData;

/* functions and macros for initializing and accessing elements of global data */

OSErr	InitGlobalData(XCmdBlockPtr param, Handle rsrc, GlobalData *gdataPtr);
ListTableType GetTable(GlobalData gdata);
ResID GetRsrcID(GlobalData gdata, int id);

#define GetParam(g)				((**(g)).param)
#define GetParamCount(g)		((**(g)).param->paramCount)
#define GetError(g)				((**(g)).error)
#define SetError(g, e)			((**(g)).error = (e))

/*------------------------------------------------------------------------*/

/* parameter conversion functions */

OSErr	param2attr(GlobalData gdata, short arg, StrListAttr *attr);
OSErr	param2cmpr(GlobalData gdata, short arg, StrCompareType *cmpr);
OSErr param2array(GlobalData gdata, short arg, ArrayType *arrayp, ArraySub *sub);
OSErr param2arrayname(GlobalData gdata, short arg, String *name);
OSErr param2subscript(GlobalData gdata, short arg, CStr255 subscript);

void	result2cmpr(GlobalData gdata, StrCompareType cmpr);

/*------------------------------------------------------------------------*/

/*	These functions are an interface between the parameters passed
	from HyperCard and the array library. The functions are
	meant to be called once the appropriate command has been identified,
	and, in most cases, after an array has been extracted from the
	parameter list. Though most functions accept subscript parameter,
	some may choose to ignore it; this was done so that the functions
	could be called in a uniform format.
	
	There is a one-to-one correspondence between these functions and
	the features of the XFCN.
*/

/* getting info about XFCN */
OSErr	ExecUsage(GlobalData gdata, short arg);
OSErr	ExecVersion(GlobalData gdata, short arg);

/* getting errors */
OSErr	ExecError(GlobalData gdata, short arg);

/* creating and disposing */
OSErr ExecNew(GlobalData gdata, short arg, Boolean setdimensions);
OSErr ExecDispose(GlobalData gdata, short arg);

/* getting and setting dimensions */
OSErr	ExecSize(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);
OSErr	ExecGetDim(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);
OSErr	ExecSetDim(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);

/* inserting and deleting items */
OSErr	ExecInsert(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);
OSErr	ExecDelete(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);

/* adding items */
OSErr	ExecAdd(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);

/* setting and getting items */
OSErr	ExecSet(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);
OSErr	ExecGet(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);

/* searching */
OSErr	ExecSearch(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);
OSErr	ExecBinSearch(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);
OSErr	ExecSeqSearch(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);

/* sorting */
OSErr	ExecSort(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);
OSErr	ExecQkSort(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);
OSErr	ExecShSort(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);

/* set operations */
OSErr ExecUnion(GlobalData gdata, short arg);
OSErr ExecDiff(GlobalData gdata, short arg);
OSErr ExecInter(GlobalData gdata, short arg);

/* stack operations */
OSErr	ExecPush(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);
OSErr	ExecPop(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);
OSErr	ExecTop(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);

/* setting and getting attributes */
OSErr	ExecSetAttr(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);
OSErr	ExecGetAttr(GlobalData gdata, short arg, ArrayType array, ArraySub *sub);

#endif

