/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

#ifndef _H_arraylib
#define _H_arraylib

#ifndef _H_strlistlib
#include "strlistlib.h"
#endif

/* characters for subscripts */
#define SUBSCRIPT_START	'['
#define SUBSCRIPT_END	']'
#define SUBSCRIPT_RANGE	''

#define MAXDIM		(1)	/* maximum number of dimensions */

typedef enum {
	SUBSCRIPT0,	/* [] */
	SUBSCRIPT1,	/* [n1] */
	SUBSCRIPT2,	/* [n1] */
	SUBSCRIPT3,	/* [n1n2] */
	SUBSCRIPT4,	/* [n2] */
	SUBSCRIPT5	/* [] */
} ArraySubType;

/* structure describing a subscript */
typedef struct {
	ListIndex lo, hi;	/* low and high ends of subscript */
	ArraySubType type;/* type of subscript */
} ArraySub;

/* structure describing an array */
typedef struct {
	short			ndim;				/* number of dimensions */
	ArraySub		dim[MAXDIM];	/* dimensions of array */
	StrListType list;				/* the one dimensional array of strings */
} ArrayStruct, **ArrayType;

char *parse_subscript(char *str, ArraySub *dim, ArraySub *sub);

StrListType array_getlist(ArrayType array);

OSErr array_new(ArrayType *arrayp);
OSErr array_dispose(ArrayType array);
OSErr array_update(ArrayType array);

ListIndex array_size(ArrayType array);
OSErr array_getdim(ArrayType array, ArraySub *sub);
OSErr array_setdim(ArrayType array, ArraySub *sub);

OSErr array_insert(ArrayType array, ArraySub *sub, String str, char sep);
OSErr array_delete(ArrayType array, ArraySub *sub);
OSErr array_add(ArrayType array, String str, char sep);

OSErr array_get(ArrayType array, ArraySub *sub, char sep, String *strp);
OSErr array_set(ArrayType array, ArraySub *sub, String str, char sep);

OSErr array_search(ArrayType array, ArraySub *sub, String str, ListIndex *index);
OSErr array_binsearch(ArrayType array, ArraySub *sub, String str, ListIndex *index);
OSErr array_seqsearch(ArrayType array, ArraySub *sub, String str, ListIndex *index);

OSErr array_sort(ArrayType array, ArraySub *sub);
OSErr array_qksort(ArrayType array, ArraySub *sub);
OSErr array_shsort(ArrayType array, ArraySub *sub);

OSErr array_union(ArrayType array1, ArrayType array2, ArrayType array3,
						ArraySub *sub1, ArraySub *sub2);
OSErr array_diff(ArrayType array1, ArrayType array2, ArrayType array3,
						ArraySub *sub1, ArraySub *sub2);
OSErr array_inter(ArrayType array1, ArrayType array2, ArrayType array3,
						ArraySub *sub1, ArraySub *sub2);
						
OSErr array_push(ArrayType array, String str, char sep);
OSErr array_pop(ArrayType array, ListIndex cnt, char sep, String *strp);
OSErr array_top(ArrayType array, ListIndex cnt, char sep, String *strp);

#endif
