/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	This function parses an expression with the following grammar:
		(note: n1 and n2 are numbers; lo and hi are dimensions of array)
	subscript ::=
		  '[' ']' | nil		{ final(lo, hi) }
		| '[' n1 ']'			{ final(n1, n1) }
		| '[' n1 '' ']'		{ final(n1, hi) }
		| '[' n1 '' n2 ']'	{ final(n1, n2) }
		| '[' '' ']'			{ final(lo, hi) }
		| '[' '' n2 ']'		{ final(lo, n2) }
		;
	number ::= digit | digit number;	
*/

#include <MyHeaders>
#include "arraylib.h"

static void setsub(ArraySub *sub, ListIndex lo, ListIndex hi, ArraySubType type);
static char *skipwhite(char *s);
static char *skipnum(char *s);
static Boolean isnum(char c);
static char *number(char *s, long *n);

#define lookahead(s)	(*((s) = skipwhite(s)))
#define match(s, c)	{(s) = skipwhite(s); if (*(s)++ != (c)) goto error;}

char *parse_subscript(char *str, ArraySub *dim, ArraySub *sub)
{
	ListIndex n1, n2;
	
	if (! lookahead(str))									/* nil */
		goto final1;
	else {
		match(str, SUBSCRIPT_START);
		if (lookahead(str) == SUBSCRIPT_END) {				/* '[' ']' */
			match(str, SUBSCRIPT_END);
			goto final1;
		}
		else if (isnum(lookahead(str))) {				/* '[' n1 */
			str = number(str, &n1);
			if (lookahead(str) == SUBSCRIPT_END) {			/* '[' n1 ']' */
				match(str, SUBSCRIPT_END);
				goto final2;
			}
			else if (lookahead(str) == SUBSCRIPT_RANGE) {	/* '[' n1 '' */
				match(str, SUBSCRIPT_RANGE);
				if (lookahead(str) == SUBSCRIPT_END) {		/* '[' n1 '' ']' */
					match(str, SUBSCRIPT_END);
					goto final3;
				}
				else if (isnum(lookahead(str))) {		/* '[' n1 '' n2 ']' */
					str = number(str, &n2);
					match(str, SUBSCRIPT_END);
					goto final4;
				}
			}
		}
		else if (lookahead(str) == SUBSCRIPT_RANGE) {		/* '[' '' */
			match(str, SUBSCRIPT_RANGE);
			if (lookahead(str) == SUBSCRIPT_END) {			/* '[' '' ']' */
				match(str, SUBSCRIPT_END);
				goto final5;
			}
			else if (isnum(lookahead(str))) {			/* '[' '' n2 ']' */
				str = number(str, &n2);
				match(str, SUBSCRIPT_END);
				goto final6;
			}
		}
	}
	
error:
	return(NULL);

#define lo	(dim->lo)
#define hi	(dim->hi)
final1:	setsub(sub, lo, hi, SUBSCRIPT0); return(str);
final2:	setsub(sub, n1-1, n1, SUBSCRIPT1); return(str);
final3:	setsub(sub, n1-1, hi, SUBSCRIPT2); return(str);
final4:	setsub(sub, n1-1, n2, SUBSCRIPT3); return(str);
final5:	setsub(sub, lo, hi, SUBSCRIPT4); return(str);
final6:	setsub(sub, lo, n2, SUBSCRIPT5); return(str);
#undef lo
#undef hi
}

static void setsub(ArraySub *sub, ListIndex lo, ListIndex hi, ArraySubType type)
{
	sub->lo = lo;
	sub->hi = hi;
	sub->type = type;
}

static char *skipwhite(char *s)
{
	while (_isspace(*s)) ++s;
	return(s);
}

static char *number(char *s, long *n)
{
	*n = atol(s);
	return(skipnum(s));
}

static Boolean isnum(char c)
{
	return(c == '-' || c == '+' || _isdigit(c));
}

static char *skipnum(char *s)
{
	if (*s == '-' || *s == '+') ++s;
	while (_isdigit(*s)) ++s;
	return(s);
}
