/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
A list table is a table containing a sorted list of names of
arrays (see the array library in "arraylib.h"),
and a corresponding list of handles to the actual arrays. For
instance, the lists may contain
	Names		Handles
	list1		3ff6e
	list2		3ff68
	list3		4000c
When a certain list is requested, its name is located
in the names list, and the handle to the list is retrieved
from the corresponding item in the handles list. The case
of letters in the names list is ignored (i.e., upper
case is the same as lower case).

This library takes care of creating and disposing of the arrays
lists that are contained in a table; thus, to create an
array simply call ltable_insert, and to dispose of it
call ltable_delete.

Brief descriptions of the functions in this library follow.

ltable_new		Allocates a new table
ltable_dispose	Disposes of an existing table and of all the arrays
					it contains handles to.
ltable_update	Updates the function pointers stored in the table.
					Should be called before a table is first used since
					function addresses on the Macintosh can be loaded into
					different locations in memory at different times during
					execution of a program.
ltable_insert	Creates and inserts a new array into the table.
					The name of the string list must not conflict with any
					existing string list.
ltable_delete	Removes the entry for the array from the table
					and disposes of the array and of its contents.
ltable_get		Finds and returns the named array.
*/

#ifndef _H_listtable
#define _H_listtable

#ifndef _H_arraylib
#include "arraylib.h"
#endif

/* structure describing a list table */
typedef struct {
	ListType	names;
	ListType	handles;
} ListTableStruct, **ListTableType;

/* creating and disposing */
OSErr	ltable_new(ListTableType *tablep);
OSErr ltable_dispose(ListTableType table);

OSErr ltable_update(ListTableType table);

/* inserting and deleting */
OSErr ltable_insert(ListTableType table, String name);
OSErr ltable_delete(ListTableType table, String name);

/* getting an array */
OSErr ltable_get(ListTableType table, String name, ArrayType *arrayp);

#endif