/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	Functions for converting a list represented as substrings into
	an array list. See header ("strlistlib.h") for more details.
*/

#include <MyHeaders>
#include "strlistlib.h"
#include "strlib.h"

static String substrdup(char *str, char separator);

static String substrdup(char *str, char separator)
{
	String		dup;
	StrIndex		length;

	length = substrlen(str, separator);
	PtrToHand(str, &dup, length+1);
	if (MemError()) return(NULL);
	(*dup)[length] = 0;
	return(dup);
}

OSErr substr2list(char *str, char separator, ListType *listPtr)
{
	ListIndex	dummyIndex;
	Boolean	dummySorted = false;
	
	*listPtr = lnew();
	if (! *listPtr) return(MemError());
	return(substrset(*listPtr, 0, str, separator, &dummyIndex, &dummySorted));
}


/*
	Notes: 1.	Any items being replaced by the substrings should either
					contain NULL pointers or else a valid string handle.
					Failure to comply with this rule will cause various
					horrendous bugs; I fell for them once!
*/
OSErr substrset(ListType list,	/* list into which to set */
					ListIndex index,	/* index at which to start */
					char *str,			/* source string */
					char separator,	/* substring separator */
					ListIndex *successful,/* number of successfully set items */
					Boolean *sorted)	/* true if list is still in sorted order */
{
	String		dup;			/* duplicate copy of current substring */
	String		lastdup;		/* last duplicate successfully set (needed for
										testing sorted condition even if dup is NULL,
										which may happen if there's an error) */
	ListIndex	i;				/* index to current item */
	ListIndex	count;		/* number of substrings */
	ListIndex	increase;	/* amount list was increased by */
	OSErr			err;			/* last error */
	
	assert(lvalidlist(list));
	assert(str != NULL);
	assert(index >= lfirst(list) && index <= lsize(list));
	*successful = 0;
	
	/* count substrings */
	count = substrcnt(str, separator);
	if (count == 0) return(noErr);
	
	/* increase size of list to accommodate all substrings */
	if (index+count > lsize(list)) {
		increase = index + count - lsize(list);
		lincrease(list, increase);
		if (MemError()) return(MemError());
	}

	/* set count strings */
	err = noErr;
	lastdup = dup = NULL;
	for (i = index; i < index+count; i++) {
		String	oldstring;
	
		/* duplicate current substring */
		dup = substrdup(str, separator);
		if (! dup) {
			OSErr err = MemError();
			ldecrease(list, MIN(increase, lsize(list)-i));
			break;
		}
	
		/* dispose of old string */
		oldstring = lget(list, i);
		if (oldstring) hfree(oldstring);
		
		/* set item to new substring */
		lset(list, i, dup);
		++*successful;
		lastdup = dup;
		
		/* check against previous item for sorted order */
		if (*sorted && i > lfirst(list) && dup && lget(list, i-1))
			*sorted = (lorder(list, dup, lget(list, i-1)) >= 0);
		
		/* advance to next substring */
		str += substrlen(str, separator)+1;
	}
	
	/* check last item for sorted order */
	if (*sorted && i < lsize(list) && lastdup && lget(list, i))
		*sorted = (lorder(list, lastdup, lget(list, i)) <= 0);
	
	return(err);
}
