/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	Main header for BinaryTree XFCN. See manual for more details.
*/

#ifndef _H_TreeMain
#define _H_TreeMain

#ifndef _H_strforest
#include "strforest.h"
#endif

/*------------------------------------------------------------------------*/
/* resources */
/*------------------------------------------------------------------------*/

/* Resource IDs are stored internally as unique integers.
	The IDs are passed to a function that maps these internal
	numbers to the IDs of the actual resources in the resource file.
*/

enum {
	RSRC_INFO_ID,			/* info strings */
	RSRC_COMMANDS_ID,		/* command words */
	RSRC_ATTRIBUTES_ID,	/* attribute words */
	RSRC_COMPARES_ID,		/* string comparison methods */
	RSRC_OUTPUTS_ID,		/* data to output when traversing tree */
	RSRC_COUNT				/* number of resources */
};

/* the info strings */
enum {
	RSRC_VERSION_INDEX=1,	/* version info */
	RSRC_USAGE_INDEX			/* usage info */
};

/*------------------------------------------------------------------------*/
/* numbers of the command strings */
/*------------------------------------------------------------------------*/

typedef enum {
	COMMAND_NULL,
	COMMAND_VERSION,
	COMMAND_USAGE,
	COMMAND_ERROR,
	COMMAND_NEW,
	COMMAND_DISPOSE,
	COMMAND_INSERT,
	COMMAND_DELETE,
	COMMAND_GET,
	COMMAND_SET,
	COMMAND_ROOT,
	COMMAND_LEFT,
	COMMAND_RIGHT,
	COMMAND_PARENT,
	COMMAND_INORDER,
	COMMAND_PREORDER,
	COMMAND_POSTORDER,
	COMMAND_LEVELORDER,
	COMMAND_RANGE,
	COMMAND_HEIGHT,
	COMMAND_GETATTR,
	COMMAND_SETATTR,
	COMMAND_LAST
} CommandType;

/*------------------------------------------------------------------------*/
/* GlobalData */
/*------------------------------------------------------------------------*/

#define TREE_CHKSUM	'TREE'

/*	Structure containing various information that must be stored
	between invocations of the code resource. A handle to this
	data is stored in a HyperCard global variable. The first
	field (chksum) is set to a unique value, which aids in detecting
	a faulty handle or a handle belonging to an incompatible version
	of the program. The forest field contains a list of the names of
	all of the trees created by the program, and the datum
	field of each node contains a handle to the tree.
	The baseID field contains the base reference ID for
	the code resource and all resources it uses. This ID is the same
	as the ID of the code resource itself. The error field contains
	the error that occured on the last call to this code resource.
	The param field contains a pointer to the parameter block
	passed from HyperCard.
	
	Most functions only make used of the param field, but all functions
	must have access to all of the fields.
*/ 
typedef struct {
	long				chksum;	/* checksum to guard against corrupted data */
	XCmdBlockPtr	param;	/* HyperCard's parameter block */
	StrForestType	forest;	/* forest of trees */
	ResID				baseID;	/* ID of code resource */
	OSErr				error;	/* last error */
	ResID				map[RSRC_COUNT]; /* resource map */
} GlobalDataStruct, **GlobalData;

/* functions and macros for initializing and accessing elements of global data */
OSErr InitGlobalData(XCmdBlockPtr param, Handle rsrc, GlobalData *gdataPtr);
StrForestType GetForest(GlobalData gdata);
ResID GetRsrcID(GlobalData gdata, int id);

#define GetParam(g)				((**(g)).param)
#define GetParamCount(g)		((**(g)).param->paramCount)
#define GetError(g)				((**(g)).error)
#define SetError(g, e)			((**(g)).error = (e))

/*------------------------------------------------------------------------*/

/* parameter conversion functions */

OSErr	param2attr(GlobalData gdata, short arg, TreeAttrType *attr);
OSErr	param2cmpr(GlobalData gdata, short arg, StrCompareType *cmpr);

void	result2cmpr(GlobalData gdata, StrCompareType cmpr);

/*------------------------------------------------------------------------*/

/*	These functions are an interface between the parameters passed
	from HyperCard and the strtree library. The functions are
	meant to be called once the appropriate command has been identified,
	and, in most cases, after the specified tree has been extracted from the
	parameter list. There is a one-to-one correspondence between these
	functions and the features of the XFCN. */

OSErr	ExecVersion(GlobalData gdata, short arg);
OSErr	ExecUsage(GlobalData gdata, short arg);

OSErr	ExecError(GlobalData gdata, short arg);

/* creating and disposing */
OSErr	ExecNew(GlobalData gdata, short arg);
OSErr	ExecDispose(GlobalData gdata, short arg);

/* inserting and deleting */
OSErr	ExecInsert(GlobalData gdata, short arg, StrTreeType tree);
OSErr	ExecDelete(GlobalData gdata, short arg, StrTreeType tree);

/* setting and getting data */
OSErr	ExecGet(GlobalData gdata, short arg, StrTreeType tree);
OSErr	ExecSet(GlobalData gdata, short arg, StrTreeType tree);

/* accessing nodes */
OSErr	ExecLeft(GlobalData gdata, short arg, StrTreeType tree);
OSErr	ExecRight(GlobalData gdata, short arg, StrTreeType tree);
OSErr	ExecParent(GlobalData gdata, short arg, StrTreeType tree);
OSErr	ExecRoot(GlobalData gdata, short arg, StrTreeType tree);

/* range searching */
OSErr	ExecRange(GlobalData gdata, short arg, StrTreeType tree);

/* traversing */
OSErr	ExecInorder(GlobalData gdata, short arg, StrTreeType tree);
OSErr	ExecPreorder(GlobalData gdata, short arg, StrTreeType tree);
OSErr	ExecPostorder(GlobalData gdata, short arg, StrTreeType tree);
OSErr ExecLevelorder(GlobalData gdata, short arg, StrTreeType tree);

/* getting height of tree */
OSErr ExecHeight(GlobalData gdata, short arg, StrTreeType tree);

/* setting and getting attributes */
OSErr	ExecGetAttr(GlobalData gdata, short arg, StrTreeType tree);
OSErr	ExecSetAttr(GlobalData gdata, short arg, StrTreeType tree);

#endif