/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	This file contains functions for comparing keys of tree nodes
	containing strings, and for disposing of tree keys and datum
	fields containing strings. These functions are used by the
	string tree library *and* by the string tree forest library.
	All of these functions are installed into the underlying
	binary tree library, which is used by both the string tree
	and string tree forest libraries.
*/

#include <MyHeaders>
#include "nodeutil.h"

/*------------------------------------------------------------------------*/
/* Disposing of keys and datum fields */
/*------------------------------------------------------------------------*/

/* dispose of a key containing a string */
int disposekey(TreeKey key)
{
	if (key)
		strfree((String) key);
	return(0);
}

/* dipose of a datum containing a string */
int disposedatum(TreeDatum datum)
{
	if (datum)
		strfree((String) datum);
	return(0);
}

/*------------------------------------------------------------------------*/
/* Comparing keys */
/*------------------------------------------------------------------------*/

/* compare two keys containing strings while searching */
int comparekeys(TreeKey key1, TreeKey key2)
{
	return(strcmp(*(String)key1, *(String)key2));
}

/* ignore case while comparing two keys containing strings */
int compareikeys(TreeKey key1, TreeKey key2)
{
	return(stricmp(*(String)key1, *(String)key2));
}

/* ignore case while ordering two keys containing strings */
int orderikeys(TreeKey key1, TreeKey key2)
{
	return(striorder(*(String)key1, *(String)key2));
}

/* use international utility rules to compare the keys */
int IUcomparekeys(TreeKey key1, TreeKey key2)
{
	int cmp;
	HandleKey state1, state2;
	
	state1 = hlock(key1, false);
	state2 = hlock(key2, false);
	cmp = IUstrcmp(*(String)key1, *(String)key2);
	hunlock(key1, state1);
	hunlock(key2, state2);
	return(cmp);
}

/* use international utility rules to order the keys */
int IUorderkeys(TreeKey key1, TreeKey key2)
{
	int cmp;
	SignedByte state1, state2;
	
	state1 = hlock(key1, false);
	state2 = hlock(key2, false);
	cmp = IUstrorder(*(String)key1, *(String)key2);
	hunlock(key1, state1);
	hunlock(key2, state2);
	return(cmp);
}

/* compare the keys numerically */
int numcmpkeys(TreeKey key1, TreeKey key2)
{
	return(floatstrcmp(*(String)key1, *(String)key2));
}

/* order the keys numerically */
int numorderkeys(TreeKey key1, TreeKey key2)
{
	return(floatstrcmp(*(String)key1, *(String)key2));
}
