/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	See header ("strforest.h") for more details.
*/

#include <MyHeaders>
#include "strforest.h"

static Boolean validforest(StrForestType forest);
static int disposetdatum(TREE_DISPOSEDATUM_PROTOTYPE);

#if ASSERTIONS
static Boolean validforest(StrForestType forest)
{
	assert(validhandle((Handle) forest));
	assert(tgetcmp(forest) == compareikeys);
	assert(tgetorder(forest) == compareikeys);
	assert(tgetdisposekey(forest) == disposekey);
	assert(tgetdisposedatum(forest) == disposetdatum);
	return(true);
}
#endif

/* create a new empty forest */
OSErr str_forestnew(StrForestType *forestptr)
{
	StrForestType	forest;
	OSErr err = noErr;
	
	*forestptr = NULL;
	forest = tnew(compareikeys, compareikeys);
	if ((err=MemError()) == noErr) {
		str_forestupdate(forest);
		*forestptr = forest;
		return(noErr);
	}
	return(err);
}

/* update any volatile fields of the forest, such as function pointers,
	which could be loaded into different locations in memory. */
void str_forestupdate(StrForestType forest)
{
	tsetcmp(forest, compareikeys);
	tsetorder(forest, compareikeys);
	tsetdisposekey(forest, disposekey);
	tsetdisposedatum(forest, disposetdatum);
	assert(validforest(forest));
}

/* dispose of the forest */
OSErr str_forestdispose(StrForestType forest)
{
	assert(validforest(forest));
	return(tdispose(forest));
}

/* dispose of a datum containing a string tree */
static int disposetdatum(TreeDatum datum)
{
	if (datum)
		return(str_tdispose((StrTreeType) datum));
	return(0);
}

/* find the node with the specified key and return its datum */
OSErr str_forestfind(StrForestType forest, StrForestKey key,
							StrForestDatum *strtree)
{
	TreeNode	node;
	
	assert(validforest(forest));
	assert(validhandle(key));
	
	*strtree = NULL;
	node = tfind(forest, (TreeKey) key, NULL);
	if (node) {
		*strtree = tdatum(node);
		str_tupdate(*strtree);
	}
	else
		return(ERR_FOREST_NOTFOUND);
	return(noErr);
}

/* insert a new node into the forest containing a copy of the given key */
OSErr str_forestinsert(StrForestType forest, StrForestKey key,
								StrCompareType cmpr)
{
	TreeNode		node;
	StrTreeType	strtree;
	OSErr			err = noErr;
	
	assert(validforest(forest));
	assert(validhandle(key));
	
	/* create the string tree for the new node */
	if (str_forestfind(forest, key, &strtree) != ERR_FOREST_NOTFOUND)
		return(ERR_FOREST_EXISTS);
	err = str_tnew(&strtree, cmpr);
	if (err == noErr) {
		/* copy the key and insert as a new node */
		key = strdup(key);
		if ((err=MemError()) == noErr) {
			node = tinsert(forest, (TreeKey) key);
			if (node) {
				/* set new node's datum to the string tree */
				tsetdatum(forest, node, (TreeDatum) strtree);
				return(noErr);
			}
			err = MemError();
			strfree(key);
		}
		str_tdispose(strtree);
	}
	return(err);
}

/* delete the node with the specified key from the forest */
OSErr	str_forestdelete(StrForestType forest, StrForestKey key)
{
	OSErr err;
	StrForestDatum dummy_strtree;
	
	err = str_forestfind(forest, key, &dummy_strtree);
	if (! err)
		tdelete(forest, (TreeKey) key);
	return(err);
}

