/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
A string forest is a tree whose keys are names of string trees
(see strtreelib) and whose datum fields are string trees.

When a certain string tree is requested, its name is located
in the forest, and the handle to the string tree is retrieved
from the corresponding datum. The case of letters in the names
of trees is ignored (i.e., upper case is the same as lower case).

This library takes care of creating and disposing of the string
trees that are contained in a forest; thus, to create a
string list simply call str_forestinsert, and to dispose of it
call str_forestdelete.

This library uses the basic binary tree library to store the
actual forest.

Brief descriptions of the functions in this library follow.

str_forestnew		Allocates a new forest
str_forestdispose	Disposes of an existing forest and of all the string
						trees it contains handles to.
str_forestupdate	Updates the function pointers stored in the forest.
						Should be called before a forest is first used since
						function addresses on the Macintosh can be loaded into
						different locations in memory at different times during
						execution of a program.
str_forestinsert	Creates and inserts a new string tree into the forest.
						The name of the string tree must not conflict with any
						existing string tree.
str_forestdelete	Removes the entry for the string tree from the forest
						and disposes of the string tree and of its contents.
str_forestfind		Finds and returns the named string tree.
*/

#ifndef _H_strforest
#define _H_strforest

#ifndef _H_strtreelib
#include "strtreelib.h"
#endif

/* data types */
typedef String			StrForestKey;
typedef StrTreeType	StrForestDatum;
typedef TreeType		StrForestType;

OSErr str_forestnew(StrForestType *forestptr);
void	str_forestupdate(StrForestType forest);
OSErr str_forestdispose(StrForestType forest);
OSErr str_forestfind(StrForestType forest, StrForestKey key,
							StrForestDatum *strtree);
OSErr str_forestinsert(StrForestType forest, StrForestKey key,
								StrCompareType cmpr);
OSErr	str_forestdelete(StrForestType forest, StrForestKey key);

#endif
