/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

#include <MyHeaders>
#include "str_tprivate.h"

/*
	This file contains functions for using the string tree's output
	buffer. This buffer is used by functions that must collect
	any number of output items, such str_tget_many (which gets
	any number of datum fields) and str_tinorder (which traverses
	the tree, and returns information about the nodes encountered).
	
	The buffer functions use the "buflib" library, which appends
	data to an ever growing handle.
	
	To use the buffer, you must first call str_tinitbuf, which
	will clear and initialize the buffer. Then, call str_toutputstr
	or str_toutputptr for each string which must be appended to
	the buffer. Each string is followed by a separator character.
	When finished, call str_tendbuf, which will dispose
	of the buffer data structure and return the handle to the
	buffer's contents. This function will also remove the
	last trailing separator character (which was set by
	the final str_toutputstr or str_toutputptr).
*/

/*----------------------------------------------------------------*/
/* output buffer functions */
/*----------------------------------------------------------------*/

OSErr str_tinitbuf(StrTreeBufType *stbuf, char separator)
{
	OutputBuffer buf;
	OSErr err = noErr;
	
	/* store separator */
	if (separator == 0)
		separator = DFLT_SEPARATOR;
	stbuf->separator = separator;
	
	/* initialize output buffer */
	err = buf_new(&buf, NULL);
	if (! err)
		stbuf->buf = buf;

	return(err);
}

/* end use of output buffer, and place contents into output */
OSErr str_tendbuf(StrTreeBufType *stbuf, String *output)
{	
	size_t len;
	
	*output = buf_detach(stbuf->buf);
	/* remove trailing separator */
	len = strlen(**output);
	if (len > 0)
		(**output)[len-1] = '\0';
	return(noErr);
}

/* append string and separator to output buffer */
OSErr str_toutputstr(StrTreeBufType *stbuf, String str)
{
	OSErr	err = noErr;
	
	if (str)
		err = buf_outputhndl(stbuf->buf, str, 0, strlen(*str));
	if (! err)
		err = buf_outputchar(stbuf->buf, stbuf->separator);
	return(err);
}

/* append string and separator to output buffer */
OSErr str_toutputptr(StrTreeBufType *stbuf, char *str)
{
	OSErr	err = noErr;
	
	if (str)
		err = buf_outputptr(stbuf->buf, str, strlen(str));
	if (! err)
		err = buf_outputchar(stbuf->buf, stbuf->separator);
	return(err);
}

