/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

#include <MyHeaders>
#include "str_tprivate.h"

/*----------------------------------------------------------------*/
/* Accessing nodes */
/*----------------------------------------------------------------*/

static OSErr copykey(String *destkeyPtr, String srckey);

/* copy source key to destination key */
static OSErr copykey(String *destkeyPtr, String srckey)
{
	*destkeyPtr = srckey;
	HandToHand(destkeyPtr);
	return(MemError());
}

/* return a copy of the key of the node to the left of the node with
	the given key, or NULL if no such node exists
*/
OSErr str_tleft(StrTreeType strtree, String key, String *leftkey)
{
	TreeNode	node;
	
	assert(str_tvalid(strtree));
	assert(str_tvalidkey(key));

	*leftkey = NULL;
	node = tfind(gettree(strtree), (TreeKey) key, NULL);
	if (node) {
		node = tleft(node);
		if (node)
			return(copykey(leftkey, (String) tkey(node)));
	}
	return(noErr);
}

/* return a copy of the key of the node to the right of the node with
	the given key, or NULL if no such node exists
*/
OSErr str_tright(StrTreeType strtree, String key, String *rightkey)
{
	TreeNode	node;
	
	assert(str_tvalid(strtree));
	assert(str_tvalidkey(key));

	*rightkey = NULL;
	node = tfind(gettree(strtree), (TreeKey) key, NULL);
	if (node) {
		node = tright(node);
		if (node)
			return(copykey(rightkey, (String) tkey(node)));
	}
	return(noErr);
}

/* return a copy of the key of the parent of the node with the given
	key, or NULL if node does not exist or if the node is the 
	root of the tree.
*/
OSErr str_tparent(StrTreeType strtree, String key, String *parentkey)
{
	TreeNode	parent, node;
	
	assert(str_tvalid(strtree));
	assert(str_tvalidkey(key));

	*parentkey = NULL;
	node = tfind(gettree(strtree), (TreeKey) key, &parent);
	if (node && parent && node != troot(gettree(strtree)))
		return(copykey(parentkey, (String) tkey(parent)));
	return(noErr);
}

/* return the a copy of the key of the root node of the tree, or NULL
	if the tree is empty.
*/
OSErr str_troot(StrTreeType strtree, String *rootkey)
{
	TreeNode	root;
	
	assert(str_tvalid(strtree));

	*rootkey = NULL;
	root = troot(gettree(strtree));
	if (root)
		return(copykey(rootkey, (String) tkey(root)));
	return(noErr);
}
