/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

#include <MyHeaders>
#include "treelib_private.h"

/*----------------------------------------------------------------*/
/* Range searching */
/*----------------------------------------------------------------*/

/*
	Executes a one-dimensional range search on the tree.
	The action function is called for each node whose key falls
	within the specified range (as determined by the compare
	function). If the action function returns a non-zero value then
	the search stops and returns the same value (this is
	useful if the action function detects an error).
	
	Whatever data and parameters don't change between recursive
	calls are placed into a RangeData structure, which is on the
	stack, so that the search function may be called recursively
	from within the action function (if the data were stored in global
	variables this would not be possible). Then, the actual recursive
	search function is called.
*/

typedef struct {
	int		(*compare)(TREE_COMPARE_PROTOTYPE);
	int		(*action)(TREE_ACTION_PROTOTYPE);
	TreeType	tree;
	TreeKey	low;
	TreeKey	high;
	void		*data;
	int		stop;
} RangeData;

static void range(RangeData *rd, TreeNode node, TreeIndex index);

int trange(TreeType tree, TreeNode node, TreeKey low, TreeKey high,
					FPTR action, void *data)
{
	RangeData rd;
	
	assert(validtree(tree));
	assert(validnode(node));
	assert(validkey(low));
	assert(validkey(high));
	assert(action != NULL);
	rd.compare = tgetcmp(tree);
	rd.tree = tree;
	rd.low = low;
	rd.high = high;
	rd.action = action;
	rd.data = data;
	rd.stop = 0;
	range(&rd, node, 1);
	return(rd.stop);
}

static void range(RangeData *rd, TreeNode node, TreeIndex index)
{
	Boolean lowcmp, highcmp;
	
	if (node) {
		lowcmp = (rd->compare(getkey(node), rd->low) >= 0);
		highcmp = (rd->compare(getkey(node), rd->high) <= 0);
		if (! rd->stop && lowcmp && lchild(node))
			range(rd, lchild(node), lindex(index));
		if (! rd->stop && lowcmp && highcmp)
			rd->stop = rd->action(rd->tree, node, index, rd->data);
		if (! rd->stop && highcmp && rchild(node))
			range(rd, rchild(node), rindex(index));
	}
}
