/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	This is the ErrorName XFCN. ErrorName takes an error code
	(an integer) and converts it into the name of an error.
	For instance, the number "1" might become "ERR_BASIC_UNKNOWN".
	This is useful since HyperCard doesn't support named
	constants. For instance, the HyperTalk command:
	
		if theErrorCode is 1 then
			answer "I don't know how to do that."
		endif
		
	is much less clear than
	
		put ErrorName(theErrorCode) into theErrorName
		if theErrorName is ERR_BASIC_UNKNOWN then
			answer "I don't know how to do that."
		endif
		
	Another reason to use the ErrorName XFCN is that the actual
	error codes assigned to an error may change, but the names
	of those errors are much less likely to change.
	
	If the error code doesn't have a name then ErrorName
	returns the error code without doing any conversion.
	
	ErrorName could easily be adapted to hold the names
	of the Macintosh toolbox errors. The "MakeRsrcs"
	project contains data files for the resources.
	By modifying (or adding to) this data, you can
	use ErrorName for any other program.
	
	ErrorName complements the ErrorString XFCN, which is
	used to get a descriptive string about the error.
	ErrorName shares the resource containing the names
	of the errors with the ErrorString XFCN (i.e.,
	both XFCN's access the same resource).
*/
		
#include <MyHeaders>
#include <SetUpA4.h>
#include "lookupTabl.h"
#include "hyperlib.h"
#include "strlib.h"
#include "rsrclib.h"

/* Resource IDs are stored internally as unique integers.
	The IDs are passed to a function that maps these internal
	numbers to the IDs of the actual resources in the resource file. */

/* resources for the program */
enum {
	RSRC_INFO_ID,		/* info strings */
	RSRC_NAMES_ID,		/* table of error names */
	RSRC_COUNT			/* number of resources */
};

/* the info strings */
enum {
	RSRC_VERSION_INDEX=1,	/* version string */
	RSRC_USAGE_INDEX			/* usage string */
};

static Boolean isnum(char *s);
static void arg2name(char *arg, char *name, ResID *map);
static ResID GetRsrcID(ResID *map, ResID internal_id);

pascal void main(XCmdBlockPtr param)
{
	CStr255 str;	/* generally useful string */
	char *arg;		/* the argument to the program */
	ResID map[RSRC_COUNT];/*map from internal resource IDs to actual resource IDs*/
	OSErr err = noErr;
	
	RememberA0();
	SetUpA4();
	err = rsrc_build_named_map(ERRORNAME_XFCN_NAME ":ResourceMap", map);
	if (! err && param->paramCount == 1) {
		HLock(param->params[0]);
		arg = *param->params[0];
		switch (*arg) {
		case '!':
			GetVerString(str, GetRsrcID(map, RSRC_INFO_ID),
										RSRC_VERSION_INDEX, __DATE__, __TIME__);
			break;
		case '?':
			GetIndString(str, GetRsrcID(map, RSRC_INFO_ID),
										RSRC_USAGE_INDEX);
			PtoCstr(str);
			break;
		default:
			arg2name(arg, str, map);
		}
		HUnlock(param->params[0]);
		PtrToHand(str, &param->returnValue, strlen(str)+1);
	}
	RestoreA4();
}

/* convert argument to error name */
static void arg2name(char *arg, char *name, ResID *map)
{
	(void) lookupTablNum(GetRsrcID(map, RSRC_NAMES_ID), atol(arg), name);
	if (! *name) strcpy(name, arg);
}

/* build a version string */
static void GetVerString(char *str, ResID id, short strnum,
									char *date, char *time)
{
	GetIndString(str, id, strnum);
	PtoCstr(str);
	strcat(strcat(strcat(strcat(str, " "), date), " "), time);
}

/* return the actual ID of the resource with the given internal ID */
static ResID GetRsrcID(ResID *map, ResID internal_id)
{
	return(map[internal_id]);
}

