/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	Functions for building a table resource.
*/

#include <MyHeaders>
#include "buildtabl.h"
#include "rsrclib.h"
#include "sortlib.h"
#include "strlib.h"

static int comparenum(void *item1, void *item2);
static int comparestr(void *item1, void *item2);

/* build a list of TABL resources, adding each resource to current rsrc file */
OSErr buildTablList(TablListType *list)
{
	short i;
	Handle rsrc;
	OSErr err = noErr;
	
	for (i = 0; list[i].id && ! err; i++) {
		rsrc = buildTabl(list[i].tabl);
		if (! rsrc) return(MemError());
		err = rsrc_add(rsrc, RSRC_TYPE, list[i].id, list[i].name, resPurgeable);
		if (err) hfree(rsrc);
	}
	return(err);
}

#if DEBUG
/* dump the table resource */
void dumptabl(Handle rsrc)
{
	short count, width, srtflg, i;
	Ptr p;
	
	/* debug */
	HLock(rsrc);
	p = *rsrc;
	printf("-- dumptabl:\n");
	count = ((short*)p)[0];
	width = ((short*)p)[1];
	srtflg = ((short*)p)[2];
	printf("rsrc = %#lx; handle size = %ld\n", rsrc, GetHandleSize(rsrc));
	printf("count = %d; width = %d; srtflg = %d\n", count, width, srtflg);
	p += 3 * sizeof(short);
	for (i = 0; i < count; i++) {
		printf("i = %d; num = %d; str = '%s'\n", i, *(short*)p, p+sizeof(short));
		p += sizeof(short) + width;
	}
	HUnlock(rsrc);
}
#endif /* DEBUG */

/* build a TABL resource */
Handle buildTabl(TablRsrcType *tabl)
{
	short count; 	/* count of entries */
	short width;	/* width of entries */
	size_t size;	/* size of resource */
	Handle rsrc;	/* the resource */
	Ptr p;			/* pointer to resource */
	short i;
	
	/* count strings and get max width */
	for (count = width = 0; tabl->entry[count].str; count++) {
		size_t len = strlen(tabl->entry[count].str)+1;
		width = MAX(width, len);
	}
	/* round up width so as to remain word-aligned */
	if (width % 2) ++width;
	/* set table's fields */
	tabl->count = count;
	tabl->width = width;
	/* sort the table by the specified field */
	if (tabl->srtfld & STRFLD)
		qksort((void*)tabl->entry, count, sizeof(TablRsrcEntry), comparestr);
	else if (tabl->srtfld & NUMFLD)
		qksort((void*)tabl->entry, count, sizeof(TablRsrcEntry), comparenum);
	/* calculate size of resource header */
	size = 3 * sizeof(short);
	/* add size of entries to get size of entire resource */
	size += (width + sizeof(short)) * count;
	/* allocate handle to hold resource */
	rsrc = halloc(size);
	if (! rsrc) return(NULL);
	HLock(rsrc);
	p = *rsrc;
	/* copy header into resource */
	((short *) p)[0] = count;
	((short *) p)[1] = width;
	((short *) p)[2] = tabl->srtfld;
	p += 3 * sizeof(short);
	/* copy items, padding each string to width */
	for (i = 0; i < count; i++) {
		size_t len = strlen(tabl->entry[i].str);
		/* set entry number */
		*(short *) p = tabl->entry[i].num;
		p += sizeof(short);
		/* copy entry string */
		strcpy(p, tabl->entry[i].str);
		p += len;
		/* pad string to width */
		while (len++ < width) *p++ = 0;
	}
	return(rsrc);
}

/* compare numbers of two table items */
static int comparenum(void *item1, void *item2)
{
	TablRsrcEntry *entry1, *entry2;
	entry1 = (TablRsrcEntry *) item1;
	entry2 = (TablRsrcEntry *) item2;
	return(entry1->num - entry2->num);
}

/* compare strings of two table items */
static int comparestr(void *item1, void *item2)
{
	TablRsrcEntry *entry1, *entry2;
	entry1 = (TablRsrcEntry *) item1;
	entry2 = (TablRsrcEntry *) item2;
	return(stricmp(entry1->str, entry2->str));
}
