#ifndef _H_RsrcTypeTABL
#define _H_RsrcTypeTABL

#ifndef _H_mystdlib
#include "mystdlib.h"
#endif

/*
DESCRIPTION
Functions for manipulating a special resource of type 'TABL'.
This resource is used for quickly locating strings or numbers
in a table. The resource consists of a header:

	Number of bytes	Contents
		2 bytes			Number of entries in table
		2 bytes			Width of each string in table
		2 bytes			Field to sort on (and other flags)

This header is followed by the entries in the table:

	Number of bytes	Contents
		2 bytes			String number (can be any value)
		Width bytes		String padded to width bytes with NULL's

Each string has the width given in the header field, and is padded
with NULL's if it was too short. The entries in the table may
be sorted either by the strings (case of letters is ignored),
by the numbers, or by no field [note: this last option isn't fully
supported].
*/

#define RSRC_TYPE	'TABL'

#define MAXTABL	(64)

typedef struct {
	short num;	/* entry number */
	char *str;	/* entry string */
} TablRsrcEntry;

/* a tabl resource */
typedef struct {
	short	count;		/* number of items */ 
	short width;		/* width of each item */
	short	srtfld;		/* field on which it's sorted */
	TablRsrcEntry entry[MAXTABL]; /* entries in the table */
} TablRsrcType;

/* sort field flags */
#define STRFLD	(1<<0)	/* sorted on string field */
#define NUMFLD	(1<<1)	/* sorted on number field */

/* a list of tabl resource to build */
typedef struct {
	ResID id;	/* id of the resource */
	char *name;	/* name of the resource */
	TablRsrcType *tabl; /* the data for the resource */
} TablListType;

OSErr buildTablList(TablListType *list);
Handle buildTabl(TablRsrcType *tabl);

#endif