/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	Glue functions which simplify the calling of a code
	resource. These functions were written for THINK C and should
	be adapted for a different development environment. When compiled
	as a code resource, the main function resides in this file. It
	does some basic initializations, including setting up register
	A4 (for access to global variables) and getting the handle
	to the code resource. It then calls the functions "codersrc_main",
	which should be the main function of your code resource. While
	debugging, the program may be compiled as an application (by
	setting the APPLICATION flag); in this case, the debugging code
	should call the function "app_main", which is also in this file.
	
	Versions of THINK C prior to 4.0 placed the address of the code
	resource into the low memory global ToolScratch. Version 4.0
	now places the address into register A0, but only if the
	standard code resource header is used. If a different header
	is used (the header is a piece of code inserted at the start
	of the code resource) then register A0 does not contain the
	address and a different method must be used. Since register
	A0 is not preserved across function calls we must store its
	value before any other code is executed. 
	
	I'm not sure how the MPW compiler organizes a code resource,
	so these functions will only work with THINK C. These functions
	are based on information provided in the THINK C 4.0 manual,
	p. 84-90.
*/
/*--------------------------------------------------------------------*/

#include <MyHeaders>
#include "setjmp.h"
#include "codelib.h"
#include "hyperlib.h"

static Boolean has128K(void);

#if APPLICATION

/* call the main function while debugging as an application */
int app_main(XCmdBlockPtr param)
{
	return(codersrc_main(param, NULL, has128K()));
}

#else

#if CUSTOM_HEADER
/* this must be the first function in the file (note: function not tested) */
static header()
{
	asm {
		DC.L	0
		/* ... custom header code ... */
		lea header, a0
		jmp main
	}
}
#endif /* CUSTOM_HEADER */

/* this must be included after the header() function */
#include "SetUpA4.h"

pascal void main(XCmdBlockPtr param);

/* jump buffer used for aborting a code resource */
jmp_buf	codersrc_abort_jump;

/* main is called from an XFCN or XCMD, sets up A4 for
	THINK C's global variables, does some other initializations,
	and then calls the code resource's main function. */
pascal void main(XCmdBlockPtr param)
{
	Handle hndl;
	
	RememberA0();
	SetUpA4();
	asm {
		_RecoverHandle
		move.l a0, hndl
	}

	#if DEBUG
		if (EnterDebugger())
			Debugger();
	#endif

	if (setjmp(codersrc_abort_jump) == 0)
		(void) codersrc_main(param, hndl, has128K());

	RestoreA4();
}

/* abort the code resource */
void codersrc_abort(void)
{
	SetUpA4();
	longjmp(codersrc_abort_jump, 1);
}

#endif

/* return true if we've got the 128K ROM */
static Boolean has128K(void)
{
	short rom, machine;
	Environs(&rom, &machine);
	return(rom >= 117); /* see pp. xii of IM-IV */
}

/*--------------------------------------------------------------------*/
/* Debugging aids.
/*--------------------------------------------------------------------*/

#if DEBUG

/* bits corresponding to keys in the key map */
#define COMMAND_KMAP	(48)
#define SHIFT_KMAP	(33)
#define BKQUOTE_KMAP	(43)

static Boolean	IsKeyDown(KeyMap kmap, short key);

static Boolean IsKeyDown(KeyMap kmap, short key)
{
	short	bit = ((key-1) % 32);
	short	word = ((key-1) / 32);
	long	mask = (1L << bit);
	long	data = kmap.Key[word];

	return((data & mask) != 0);
}

/* Return true if shift key is down. */
Boolean EnterDebugger(void)
{
	KeyMap	kmap;
	
	GetKeys(&kmap);
	return(IsKeyDown(kmap, SHIFT_KMAP));
}
#endif /* DEBUG */

