/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
The function exec_pascal_xcmd loads and executes a code resource,
and returns true if it could find the code resource, or false if
it couldn't. This function has the advantage that the resource is
not loaded and then copied, as HyperCard would normally do. Thus,
calling an XCMD, even if it's large, involves less overhead
in time wasted copying the XCMD. Unfortunately, due to the
Macintosh's brain-dead memory manager, this function must
MoveHHi() the code resource and then lock it. This means that
we usually still end up copying the code resource at least once.
	
	Warning:
		Care must be taken to ensure that the stack
		containing the code resource is not closed during
		the execution of the resource. This could occur,
		for instance, if the code resource sent a message
		to HyperCard, and the handler in HyperCard went
		to another stack. If your code resource sends
		HyperCard any messages that may allow HyperCard
		to close the stack containing the resource, then
		you should not use this function, or you should
		modify it to do call DetachResource().

By writing my own glue functions in assembly language to ToolBox traps
I was able to eliminate the space wasted by THINK C's inclusion of the
MacTraps library. This resulted in a decrease in size from over 1.2K
down to a mere 190 bytes.
*/

#include <MyHeaders>
#include <pascal.h>
#include "codelib.h"

typedef void (*XCMDPtr)(XCmdBlockPtr); /* pointer to an xcmd */

static Handle get_resource(ResType type, ResID id);
static Handle get_named_resource(ResType type, StrPtr name);

/* note: the ID parameter is currently ignored */
Boolean exec_pascal_xcmd(XCmdBlockPtr param, ResType type, ResID id, StrPtr name)
{
	Handle rsrc; /* handle to code resource */
	HandleKey key; /* key for unlocking code resource */
	XCMDPtr xcmd; /* pointer to the xcmd */
	
	rsrc = get_named_resource(type, name);
	if (rsrc) {
		key = hlock(rsrc, true);
		xcmd = (XCMDPtr) *rsrc;
		CallPascal(param, xcmd);
		hunlock(rsrc, key);
	}
	return(rsrc != NULL);
}

static Handle get_named_resource(ResType type, StrPtr name)
{
	asm {
		; get resource by name
		subq.l #4, a7
		move.l type, -(a7)
		move.l name, -(a7)
		_GetNamedResource
		move.l (a7)+, d0
	}
}

#if NOT_USED /* this function works but isn't needed */
static Handle get_resource(ResType type, ResID id)
{
	asm {
		; get resource by ID
		subq.l #4, a7
		move.l type, -(a7)
		move.w id, -(a7)
		_GetResource
		move.l (a7)+, d0
	}
}
#endif
