/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	A bunch of simple functions that are useful while writing
	XCMD's. There are functions for setting and getting a global
	variable as a number or string, and similar functions for
	getting specific parameters and setting the return value. There
	are also a few other things in here, such as getting error
	strings.
*/

#include <MyHeaders>
#include "hyperlib.h"
#include "strlib.h"
#include "codelib.h"

/*--------------------------------------------------------------------*/
/* Get an error string.
/*--------------------------------------------------------------------*/

/* build a version string */
void GetVerString(char *str, ResID id, short strnum, char *date, char *time)
{
	GetIndString(str, id, strnum);
	PtoCstr(str);
	strcat(strcat(strcat(strcat(str, " "), date), " "), time);
}

/*--------------------------------------------------------------------*/
/* HyperCard global variable handling.
/*--------------------------------------------------------------------*/

long GetGlobalToNum(XCmdBlockPtr param, char *globalName)
{
	CStr255	str;
	return(atol(GetGlobalToString(param, globalName, str)));
}

void SetGlobalToNum(XCmdBlockPtr param, char *globalName, long num)
{
	CStr31	str;
	SetGlobalToString(param, globalName, ltoa(num, str));
}
	
char *GetGlobalToString(XCmdBlockPtr param, char *globalName, CStr255 str)
{
	Handle	globalValue;
	Str255	name;
	
	*str = 0;
	CtoPstr(strcpy((char*)name, globalName));	
	globalValue = GetGlobal(param, name);
	if (globalValue) {
		strncat(str, *globalValue, sizeof(CStr255)-1);
		DisposHandle(globalValue);
	}
	return(str);
}

void SetGlobalToString(XCmdBlockPtr param, char *globalName, char *str)
{
	Handle	globalValue;
	Str255	name;
	
	CtoPstr(strcpy((char*)name, globalName));
	if (PtrToHand(str, &globalValue, strlen(str)+1) == noErr) {
		SetGlobal(param, name, globalValue);
		DisposHandle(globalValue);
	}
	else
		SetGlobal(param, name, NULL);
}

/*--------------------------------------------------------------------*/
/* Parameter extraction functions.
/*--------------------------------------------------------------------*/

OSErr param2handle(XCmdBlockPtr param, short n, String *str)
{
	if (n >= param->paramCount) return(ERR_BASIC_ARGCNT);
	*str = param->params[n];
	return(noErr);
}

OSErr param2bool(XCmdBlockPtr param, short n, Boolean *bool)
{
	CStr31 str;
	
	*str = 0;
	if (n >= param->paramCount) return(ERR_BASIC_ARGCNT);
	CtoPstr(strncat(str, *param->params[n], sizeof(CStr31)-1));
	*bool = StrToBool(param, (Str31 *) str);
	return(noErr);
}

OSErr param2short(XCmdBlockPtr param, short n, short *num)
{
	long	tmpnum;
	OSErr	err;
	
	err = param2long(param, n, &tmpnum);
	if (! err)
		*num = tmpnum;
	return(err);
}

OSErr param2long(XCmdBlockPtr param, short n, long *num)
{
	HandleKey key;
	
	if (n >= param->paramCount) return(ERR_BASIC_ARGCNT);
	key = hlock(param->params[n], false);
	*num = atol(*param->params[n]);
	hunlock(param->params[n], key);
	return(noErr);
}

OSErr	param2word(XCmdBlockPtr param, short arg, short n, CStr255 str)
{
	char		*s;
	StrIndex	i;
	
	if (arg >= param->paramCount) return(ERR_BASIC_ARGCNT);
	*str = 0;
	s = *param->params[arg];
	while (*s && n > 0) {
		while (_isspace(*s)) ++s;
		while (*s && ! _isspace(*s)) ++s;
		n--;
	}
	while (_isspace(*s)) ++s;
	if (! *s) return(ERR_BASIC_USAGE);
	for (i = 0; i < sizeof(CStr255)-1 && *s && ! _isspace(*s); ++i, ++s) {
		if (*s == '\\' && s[1])
			++s;
		str[i] = *s;
	}
	str[i] = 0;
		
	return(noErr);
}

/* convert parameter into a separator character */
OSErr param2separator(XCmdBlockPtr param, short arg, char *separator)
{
	if (arg >= param->paramCount) return(ERR_BASIC_ARGCNT);
	*separator = **(param->params[arg]);
	return(noErr);
}

/*--------------------------------------------------------------------*/
/* Result setting functions.
/*--------------------------------------------------------------------*/

void result2handle(XCmdBlockPtr param, Handle hndl)
{
	param->returnValue = hndl;
}

void result2string(XCmdBlockPtr param, char *str)
{
	result2handle(param, stralloc(str));
}

void result2num(XCmdBlockPtr param, long num)
{
	CStr31	str;
	result2string(param, ltoa(num, str));
}

void result2bool(XCmdBlockPtr param, Boolean flag)
{
	CStr31	str;
	BoolToStr(param, flag, (Str31 *) str);
	result2string(param, str);
}

void result2error(XCmdBlockPtr param, OSErr errnum)
{
	result2num(param, errnum);
}
