/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

#ifndef _H_hyperlib
#define _H_hyperlib

#ifndef _H_mystdlib
#include "mystdlib.h"
#endif

#ifndef _H_errorlib
#include "errorlib.h"
#endif

/* The following definitions are used to identify XFCNs and XCMDs
	that I've written. By placing all of the definitions in one file,
	I reduce the risk of duplicating names and IDs. I can also enforce
	a standard format, instead of using different formats in each
	program I write.
	
	When naming resources used by any of these XFCNs, I follow the
	convention that the name of the XFCN precedes the name
	of the resource. For instance,
		"ArrayList:Usage"
		"BinaryTree:Functions"
		"Research:Busy:1"
	The names may be formed by concatenating the name of the XFCN
	with the name of the resource. This method reduces the risk of
	conflicting resource names between my XFCNs and XFCNs written
	by other people. It also makes it easy to find the resources
	used by this XFCN, including what they are used for, even
	if the resource file being examined contains many resources.
*/
	
/* Names of XFCNs and XCMDs I've written. Some names have
	synonyms, and have a number appended to them (the synonyms
	are used for naming the resources and, perhaps, for allowing
	slightly different names when called from HyperCard).
	The names with a "_PASCAL" appended are Pascal string variants
	of the same name, and are used for loading the code resource. */

#define ARRAYLIST_XFCN_NAME				"ArrayList"
#define ARRAYLIST_XFCN_NAME_PASCAL		((StrPtr)"\pArrayList")

#define BINARYTREE_XFCN_NAME				"BinaryTree"
#define BINARYTREE_XFCN_NAME_PASCAL		((StrPtr)"\pBinaryTree")

#define RESEARCH_XFCN_NAME					"Research"
#define RESEARCH_XFCN_NAME_PASCAL		((StrPtr)"\pResearch")

#define ERRORSTRING_XFCN_NAME				"ErrorString"
#define ERRORSTRING_XFCN_NAME_PASCAL	((StrPtr)"\pErrorString")

#define ERRORNAME_XFCN_NAME				"ErrorName"
#define ERRORNAME_XFCN_NAME_PASCAL		((StrPtr)"\pErrorName")

/* types of resources storing the actual code for the above XFCNs */
#define ARRAYLIST_XFCN_TYPE		'PROC'
#define RESEARCH_XFCN_TYPE			'PROC'
#define BINARYTREE_XFCN_TYPE		'PROC'
#define ERRORSTRING_XFCN_TYPE		'XFCN'
#define ERRORNAME_XFCN_TYPE		'XFCN'

/* IDs of the above resources */
#define ARRAYLIST_XFCN_ID			(2300)
#define RESEARCH_XFCN_ID			(2400)
#define BINARYTREE_XFCN_ID			(2500)
#define ERRORSTRING_XFCN_ID		(2600)
#define ERRORNAME_XFCN_ID			(2610)

/* names of HyperTalk global variables used by the above programs */
#define ARRAYLIST_GLOBAL_NAME		"_ArrayListGlobal"
#define BINARYTREE_GLOBAL_NAME	"_BinaryTreeGlobal"
#define RESEARCH_GLOBAL_NAME		"_ResearchGlobal"

/* types describing a HyperCard field */
typedef enum {
	BackgroundField, CardField, FieldName, FieldNumber, FieldID
} HCFieldType;

/* structure describing a HyperCard field */
typedef struct {
	HCFieldType	type;		/* type of output field (background or card) */
	HCFieldType	idtype;	/* method of specifying output field */
	union {					/* name, number, or id of field */
		String	name;
		short		number;
		short		id;
	} u;
} HCField;

void GetVerString(char *str, ResID id, short strnum, char *date, char *time);

/* Functions for setting and getting the values of HyperCard global
	variables.
*/
void SetGlobalToNum(XCmdBlockPtr param, char *globalName, long num);
void SetGlobalToString(XCmdBlockPtr param, char *globalName, char *str);
long GetGlobalToNum(XCmdBlockPtr param, char *globalName);
char *GetGlobalToString(XCmdBlockPtr param, char *globalName, CStr255 str);

/* Extract the indicated parameter from the parameter block */
OSErr	param2bool(XCmdBlockPtr param, short n, Boolean *bool);
OSErr	param2short(XCmdBlockPtr param, short n, short *num);
OSErr	param2long(XCmdBlockPtr param, short n, long *num);
OSErr	param2handle(XCmdBlockPtr param, short n, String *str);
OSErr	param2word(XCmdBlockPtr param, short arg, short n, CStr255 str);
OSErr param2separator(XCmdBlockPtr param, short arg, char *separator);

/* Set the parameter block's returnValue to the given value.
	Each data type is translated into a handle to string, except
	for the function result2handle, which sets the returnValue
	to the given handle.
*/
void	result2handle(XCmdBlockPtr param, Handle hndl);
void	result2string(XCmdBlockPtr param, char *str);
void	result2num(XCmdBlockPtr param, long n);
void	result2bool(XCmdBlockPtr param, Boolean flag);
void result2error(XCmdBlockPtr param, OSErr err);

#endif
