/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	Functions for getting information about resources.
*/

#include <MyHeaders>
#include "rsrclib.h"

/*	This function is used to build a mapping from the internal
	resource IDs used by a program to the IDs of the actual
	resources stored on disk.
	
	The names and types of resources are stored in a 'STR#' resource.
	Each string starts with the type of the resource, which is
	followed by a separator character (usually a comma), which is
	followed by the name of the resource. The function places the
	IDs of the named resources into the MAP array.

	The MAP parameter should have as many items as there
	are strings in the resource. The MAPID parameter
	gives the ID of the resource containing the map (ie,
	the STR# resource).
	
	By using an existing resource type and doing some minimal
	string processing, I avoided the trouble of writing
	routines to read in a non-standard resource and for
	editing the resource. */
OSErr rsrc_build_map(ResID mapid, ResID *map)
{
	short i;				/* index to resource map */
	CStr255 str;		/* current string from resource map */
	CStr255 name;		/* name of resource */
	ResType type;		/* type of resource */
	OSErr err = noErr;
	
	for (i = 0; ! err; i++) {
		GetIndString(str, mapid, i + 1);
		PtoCstr(str);
		err = ResError();
		if (strlen(str) == 0)
			break;
		if (! err) {
			type = *(ResType*)str;
			strcpy(name, str + sizeof(ResType) + 1);
			err = rsrc_name_to_id(type, name, &map[i]);
		}
	}
	return(err);
}

/* This function is the same as rsrc_build_map, except that it
	takes the name of the resource containing the map, instead
	of the resource's ID */
OSErr rsrc_build_named_map(char *name, ResID *map)
{
	OSErr err;
	ResID id;
	
	err = rsrc_name_to_id('STR#', name, &id);
	if (! err)
		err = rsrc_build_map(id, map);
	return(err);
}

/* convert the name of a resource into the resource's id */
OSErr rsrc_name_to_id(ResType type, char *name, ResID *id)
{
	Boolean load;
	CStr255 str;
	Handle rsrc;
	OSErr err = noErr;
	
	assert(strlen(name) < sizeof(CStr255));
	load = ResLoad;
	SetResLoad(false);
	rsrc = GetNamedResource(type, CtoPstr(strcpy(str, name)));
	err = ResError();
	SetResLoad(true);
	if (! err) {
		if (! rsrc)
			err = resNotFound;
		else
			err = rsrc_get_id(rsrc, id);
	}
	return(err);
}

/* get the id of the resource */
OSErr rsrc_get_id(Handle rsrc, ResID *id)
{
	ResType type;
	Str255 name;
	GetResInfo(rsrc, id, &type, name);
	return(ResError());
}
