/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	shell sort. Following are the profiler's output when run
	on a standard Macintosh Plus. The swap functions were compiled
	in assembly language.

	 Shsort, random data, 4096 integers
	Function          Minimum  Maximum  Total   Average    %   Entries
	swap_short             0      538 19331988      118    49   163363
	compare                0      657 19759566       95    50   206356
	Total time = 39092246 (49.905161 seconds)
	List was sorted succesfully
	
	 Shsort, sorted data, 4096 integers
	Function          Minimum  Maximum  Total   Average    %   Entries
	compare                0      505  4365267       96    99    45057
	Total time = 4365857 (5.573453 seconds)
	List was sorted succesfully
*/

#include <MyHeaders>
#include "sortlib.h"

#define get(i)	((void*) ((char*)base + size * (i)))

static void _shsort(void *base, size_t nmemb, size_t size,
							FPTR compar, VFPTR swap);

/* adapted from pp. 58 of K&R (1st edition) */
void shsort(void *base, size_t nmemb, size_t size, FPTR compar)
{
	_shsort(base, nmemb, size, compar, getswap(size));
}

static void _shsort(register void *base,
						register size_t nmemb,
						register size_t size,
						register FPTR compar,
						register VFPTR swap)
{
	register long gap, j, i; /* must use signed values */
	
	for (gap = nmemb/2; gap > 0; gap >>= 1)
		for (i = gap; i < nmemb; i++)
			for (j=i-gap; j >= 0 && compar(get(j), get(j+gap)) > 0; j-=gap)
				swap(size, get(j), get(j+gap));
}
