/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	A library of very useful functions for strings.
	Some functions are guarenteed not to move or
	purge memory, which simplifies calling them
	when the strings are contained in handles
	(there's no need to lock the handles first).
	Those functions that won't affect memory
	are marked "Memory Safe".
*/
#ifndef _H_strlib_
#define _H_strlib_

#ifndef _H_mystdlib_
#include "mystdlib.h"
#endif

typedef enum {
	STR_COMPARE_NULL,
	STR_COMPARE_EXACT,			/* strings must match exactly */
	STR_COMPARE_IGNORECASE,		/* character case is ignored */
	STR_COMPARE_INTERNATIONAL,	/* international rules are used */
	STR_COMPARE_FLOAT,			/* compare as floating point numbers */
	STR_COMPARE_LONG,				/* compare as long int numbers */
	STR_COMPARE_LAST
} StrCompareType;

double	atof(char *s);
long		atol(char *s);
char		*ltoa(long num, char *str);

size_t	substrlen(char *str1, char delim); /* Memory Safe */
int		substrcmp(char *str1, char *str2, char delim); /* Memory Safe */
int		substricmp(char *str1, char *str2, char delim); /* Memory Safe */
int		substriorder(char *str1, char *str2, char delim); /* Memory Safe */
size_t	substrcnt(register char *str, char separator); /* Memory Safe */
int		IUstrcmp(char *str1, char *str2);
int		IUstrorder(char *str1, char *str2);

char		*stristr(char *s1, char *s2); /* Memory Safe */
int		floatcmp(char *s1, char *s2, Boolean order); /* Memory Safe */


String	stralloc(char *str);
String	strdup(String str);

#define floatstrcmp(s1, s2)	floatcmp(s1, s2, false)
#define floatstrorder(s1, s2)	floatcmp(s1, s2, true)

#define strorder(str1, str2)	strcmp((str1), (str2))
#define stricmp(str1, str2)	substricmp((str1), (str2), 0)
#define striorder(str1, str2)	substriorder((str1), (str2), 0)
#define strfree(str)				hfree(str)
#endif
