/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	Main program for building TABL resources for my XFCN's.
	There's a separate file containing tables of the resources
	for each XFCN. This main routine simply queries the user
	on what program to update, opens the appropriate resource
	file, and calls the build table list function to create
	the resources from the data.
*/

#include <MyHeaders>
#include "MakeRsrcs.h"
#include "strlib.h"

void main(void);
static int Confirm(char *msg);
static void Message(char *msg);
static OSErr build(char *file, TablListType *tabls);
static void execute(void);
char *getline(char *);

void main(void)
{
	InitGraf(&thePort);
	InitFonts();
	FlushEvents(everyEvent, 0);
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);
	InitCursor();
	execute();
}

static void execute(void)
{
	struct {
		char *prog;	/* name of program (for prompt) */
		char *file;	/* name of file to put resources into */
		TablListType *tabls; /* list of TABL resources */
	} list[6] = {
		{ "ArrayList XFCN", ":ArrayList:ArrayListPROC..rsrc", list_tabls, },
		{ "BinaryTree XFCN", ":BinaryTree:BinaryTreePROC..rsrc", tree_tabls, },
		{ "Research XFCN", ":Research:ResearchPROC..rsrc", rsrch_tabls, },
		{ "ErrorString XFCN", ":Error:ErrorString..rsrc", error_string_tabls, },
		{ "ErrorName XFCN", ":Error:ErrorName..rsrc", error_name_tabls, },
		{ 0, 0, 0, },
	};
	short i;
	CStr255 msg;
	CStr255 str;
	OSErr err = noErr;
	
	for (i = 0; list[i].prog && ! err; i++) {
		strcpy(msg, "Make 'TABL' resources for ");
		strcat(msg, list[i].prog);
		strcat(msg, "?");
		if (Confirm(msg) == 1/*ok*/)
			err = build(list[i].file, list[i].tabls);
	}
	if (err) {
		ltoa(err, str);
		strcat(strcat(strcpy(msg, "Error #"), str), " occurred.");
	}
	else
		strcpy(msg, "Build was succesful.");
	Message(msg);
	
}

static void Message(char *msg)
{
	CStr255 str;
	
	ParamText(CtoPstr(strcpy(str, msg)), "\p", "\p", "\p");
	(void) Alert(129, NULL);
}

static int Confirm(char *msg)
{
	CStr255 str;
	
	ParamText(CtoPstr(strcpy(str, msg)), "\p", "\p", "\p");
	return(Alert(128, NULL));
}

static OSErr build(char *file, TablListType *tabls)
{
	CStr255 str;
	FileRef ref;
	OSErr err;
	
	ref = OpenResFile(CtoPstr(strcpy(str, file)));
	if (ResError()) return(ResError());
	err = buildTablList(tabls);
	CloseResFile(ref);
	if (! err) err = ResError();
	return(err);
}