/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	Main file for the Research XFCN. See manual for more info on use
	and program's organization.
*/

#include <MyHeaders>
#include "RsrchMain.h"
#include "lookuptabl.h"
#include "codelib.h"

/*--------------------------------------------------------------------*/
/* Prototypes
/*--------------------------------------------------------------------*/

static OSErr Initialize(XCmdBlockPtr param, Handle rsrc,
								Boolean has128k, GlobalData *gdataPtr);
static OSErr Execute(GlobalData gdata);
static OSErr str2command(GlobalData gdata, char *str, CommandType *command);

/*--------------------------------------------------------------------*/
/* The main and initialization functions
/*--------------------------------------------------------------------*/

int codersrc_main(XCmdBlockPtr param, Handle rsrc, Boolean has128k)
{	
	GlobalData gdata; /* global data used by the program */
	OSErr err;

	err = Initialize(param, rsrc, has128k, &gdata);
	if (! err)
		err = Execute(gdata);
	return(0);
}

static OSErr Initialize(XCmdBlockPtr param,
								Handle rsrc,
								Boolean has128k,
								GlobalData *gdataPtr)
{	
	if (! has128k)
		return(ERR_BASIC_ROMVER);
	result2handle(param, NULL);
	return(InitGlobalData(param, rsrc, gdataPtr));
}

/*--------------------------------------------------------------------*/
/* Functions for executing the specified command
/*--------------------------------------------------------------------*/

static OSErr Execute(GlobalData gdata)
{
	CStr255 word; /* word containing command */
	CommandType command; /* command to execute */
	OSErr err = noErr;

	command = COMMAND_NULL;
	if (GetParamCount(gdata) == 1) {
		/* a special command (not a search pattern) */
		
		/* get command word */
		err = param2word(GetParam(gdata), 0, 0, word);
		if (! err)
			err = str2command(gdata, word, &command);
		
		/* execute command */
		if (! err) {
			switch (command) {
			case COMMAND_USAGE:		err = ExecUsage(gdata, 0);		break;
			case COMMAND_VERSION:	err = ExecVersion(gdata, 0);	break;
			case COMMAND_MATCHED:	err = ExecMatched(gdata, 0);	break;
			case COMMAND_ERROR:		err = ExecError(gdata, 0);		break;
			default:						err = ERR_BASIC_COMMAND;		break;
			}
		}
		if (err)
			result2error(GetParam(gdata), err);
	}
	else
		err = ExecSearch(gdata, 0);

	/* set error code */
	if (command != COMMAND_ERROR)
		SetError(gdata, err);
	return(err);
}

/*--------------------------------------------------------------------*/

static OSErr str2command(GlobalData gdata, char *str, CommandType *command)
{
	OSErr		err;
	short		index;

	*command = COMMAND_NULL;
	err = lookupTablStr(GetRsrcID(gdata, RSRC_COMMANDS_ID), str, &index);
	if (err) return(err);
	if (index <= COMMAND_NULL || index >= COMMAND_LAST)
		return(ERR_BASIC_COMMAND);
	*command = index;
	return(noErr);
}

OSErr str2option(GlobalData gdata, char *str, OptionType *option)
{
	OSErr		err;
	short		index;

	*option = OPTION_NULL;
	err = lookupTablStr(GetRsrcID(gdata, RSRC_OPTIONS_ID), str, &index);
	if (err) return(err);
	if (index <= OPTION_NULL || index >= OPTION_LAST)
		return(ERR_SRCH_OPTION);
	*option = index;
	return(noErr);
}

