/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	Main header for Research XFCN. See manual for more details.
*/

#ifndef _H_RsrchMain
#define _H_RsrchMain

#ifndef _H_hyperlib
#include "hyperlib.h"
#endif

/*------------------------------------------------------------------------*/
/* resources */
/*------------------------------------------------------------------------*/

/* Resource IDs are stored internally as unique integers.
	The IDs are passed to a function that maps these internal
	numbers to the IDs of the actual resources in the resource file.
*/

enum {
	RSRC_INFO_ID,		/* info strings */
	RSRC_COMMANDS_ID,	/* command words */
	RSRC_OPTIONS_ID,	/* option words */
	RSRC_BUSY_CURS_0,	/* first busy cursor */
	RSRC_BUSY_CURS_1,	/* second busy cursor */
	RSRC_BUSY_CURS_2,	/* third busy cursor */
	RSRC_BUSY_CURS_3,	/* fourth busy cursor */
	RSRC_COUNT			/* number of resources */
};

#define NBUSY_CURS			(4)	/* number of busy cursors */

/* the general strings (for STR# resource) */
enum {
	RSRC_VERSION_INDEX=1,	/* version info */
	RSRC_USAGE_INDEX,			/* usage info */
	AT_BEFORE_ID /* " at or before char " (this might not be used at all) */
};

/*------------------------------------------------------------------------*/
/* numbers of the command strings */
/*------------------------------------------------------------------------*/

typedef enum {
	COMMAND_NULL,
	COMMAND_VERSION,
	COMMAND_USAGE,
	COMMAND_ERROR,
	COMMAND_MATCHED,
	COMMAND_LAST
} CommandType;

/*------------------------------------------------------------------------*/
/* numbers of the option strings */
/*------------------------------------------------------------------------*/

typedef enum {
	OPTION_NULL,

	OPTION_CHUNK,
	OPTION_GLOBAL,

	OPTION_FIRST,
	OPTION_MATCH,

	OPTION_COUNT,
	OPTION_NUMBER,
	OPTION_INVERT,
	OPTION_IGNRCASE,
	OPTION_INTERNATIONAL,

	OPTION_LIST,
	OPTION_CONTAINERS,

	OPTION_MPWMETA,
	OPTION_NOMETA,
	OPTION_UNIXMETA,

	OPTION_SEPARATOR,

	OPTION_FIRST_CARD,
	OPTION_LAST_CARD,
	OPTION_START_CARD,
	OPTION_START_CHAR,
	OPTION_START_CONTAINER,
	
	OPTION_NOPERIODIC,
	OPTION_NOCANCEL,
	OPTION_NOSPIN,
	
	OPTION_LAST
} OptionType;

/*------------------------------------------------------------------------*/
/* GlobalData */
/*------------------------------------------------------------------------*/

#define SRCH_CHKSUM	'SRCH'

/*	Structure containing various information that must be stored
	between invocations of the code resource. A handle to this
	data is stored in a HyperCard global variable. The first
	field (chksum) is set to a unique value, which aids in detecting
	a faulty handle or a handle belonging to an incompatible version
	of ArrayList. The baseID field contains the base reference ID for
	the code resource and all resources it uses. This ID is the same
	as the ID of the code resource itself. The error field contains
	the error that occured on the last call to this code resource.
	The param field contains a pointer to the parameter block
	passed from HyperCard. The where field is set to the index
	into the pattern string at which the last error occured (this
	makes figuring out what's wrong with a pattern a lot easier).
	
	Most functions only make used of the param field, but all functions
	must have access to all of the fields.
*/ 
typedef struct {
	long				chksum;	/* checksum to guard against corrupted data */
	XCmdBlockPtr	param;	/* HyperCard's parameter block */
	ResID				baseID;	/* ID of code resource */
	OSErr				error;	/* last error */
	Boolean			matched;	/* true if last search matched something */
	size_t			where;	/* index into pattern where error occurred */
	Boolean			hasWNE;	/* we have the WaitNextEvent trap */
	Boolean			inbkgnd;	/* we're running in the background */
	ResID				map[RSRC_COUNT]; /* resource map */
} GlobalDataStruct, **GlobalData;

/* functions for indirectly accessing elements of global data */

OSErr InitGlobalData(XCmdBlockPtr param, Handle rsrc, GlobalData *gdataPtr);
ResID GetRsrcID(GlobalData gdata, int id);

/* macros replacing a few simple global data functions */
#define GetParam(gdata)				((**(gdata)).param)
#define GetParamCount(gdata)		((**(gdata)).param->paramCount)
#define GetError(gdata)				((**(gdata)).error)
#define SetError(gdata, e)			((**(gdata)).error = (e))
#define GetMatched(gdata)			((**(gdata)).matched)
#define SetMatched(gdata, m) 		((**(gdata)).matched = (m))
#define GetErrorWhere(gdata)		((**(gdata)).where)
#define SetErrorWhere(gdata, w)	((**(gdata)).where = (w))
#define HasWNE(gdata)				((**(gdata)).hasWNE)
#define SetHasWNE(gdata, x)		((**(gdata)).hasWNE = (x))
#define InBackground(gdata)		((**(gdata)).inbkgnd)
#define SetInBackground(gdata,x)	((**(gdata)).inbkgnd = (x))

/*------------------------------------------------------------------------*/

OSErr	str2option(GlobalData gdata, char *str, OptionType *option);

/*------------------------------------------------------------------------*/

/*	These functions are an interface between the parameters passed
	from HyperCard and actual execution of the XFCN's features.
*/
OSErr	ExecUsage(GlobalData gdata, short arg);
OSErr	ExecVersion(GlobalData gdata, short arg);

OSErr	ExecError(GlobalData gdata, short arg);

OSErr ExecMatched(GlobalData gdata, short arg);

OSErr	ExecSearch(GlobalData gdata, short arg);

#endif

