/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	Header for Boyer-Moore search library.
*/

#ifndef _H_bmslib
#define _H_bmslib

#ifndef _H_mystdlib
#include "mystdlib.h"
#endif

/* Max size of next table. If the table overflows then we simply
	don't use the next table: the search will still work, and
	will be almost as efficient (but running time will not
	necessarily be linear). */
#define MaxNextTable	(256)

typedef uchar NextTable[MaxNextTable];
typedef uchar SkipTable[MaxNextTable];

/* Structure describing an initialized Boyer-Moore pattern */
typedef struct {
	NextTable	next;			/* next table */
	SkipTable	skip;			/* skip table */
	StrIndex		len;			/* length of pattern string */
	uchar			*pat;			/* the pattern string */
	Boolean		ignrcase;	/* if true then ignore case */
} BoyerMooreStruct, *BoyerMoore;

void		bmsinit(BoyerMoore bmdata, uchar *pat, Boolean ignrcase);
StrIndex	bmsmatch(BoyerMoore bmdata, uchar *str, StrIndex len);

#endif