/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

#ifndef _H_patlib
#define _H_patlib

#ifndef _H_mystdlib
#include "mystdlib.h"
#endif

#ifndef _H_hyperlib
#include "hyperlib.h" /* for basic error codes */
#endif

#define MAXTAG	(9)	/* max number of tagged expressions */
#define MAXPAT	(256)	/* max length of a regular expression */

typedef short		PatItem;		/* item in a compiled pattern */
typedef short		PatIndex;	/* index to an item in a pattern */
typedef PatIndex	PatState;	/* a state transition */
typedef unsigned short PatReps;	/* number of repetitions for closures */

/* state types */
typedef enum {
	PAT_NULL_STATE,		/* a state with two exits */
	PAT_ORNULL_STATE,		/* a null state used with OR during parsing */
	PAT_LITERAL_STATE,	/* a state with an item and one exit */
	PAT_CLITER_STATE,		/* iteration step of a closure */
	PAT_CLREPS_STATE		/* initialization step for a closure */
} PatStateType;

/* Structure describing a compiled regular expression. The finite state
	machine which is described by this structure is composed of
	at most MAXPAT states. Each state may have different types of
	information stored in it. A literal item, such as a letter
	or digit, requires storage for the literal and storage for
	the next state to execute (stored in the 'item' and 'next' items).
	A null state  as well as the closure iterate states  requires
	storage for the two next states to execute (stored in the 'next1'
	and 'next2' items). Finally, a closure limits state stores the
	minimum and maximum repetitions for the closure. A complete set of
	functions for setting and getting the items in this structure is
	provided, so this structure should never be accessed directly.
*/
typedef struct {
	Boolean ignrcase;	/* if true then ignore case */
	struct {
		PatStateType	type;	/* type of data stored in this state */
		union {
			PatItem	item;		/* a literal such as a letter or digit */
			PatState	next1;	/* first state for a null state */
			PatReps	min;		/* minimum repetitions for a closure state */
		} a;
		union {
			PatState	next;		/* next state for an item state */
			PatState	next2;	/* second state for a null state */
			PatReps	max;		/* maximum repetitions for a closure state */
		} b;
	} state[MAXPAT];
} PatTypeStruct, *PatType;

/* Structure describing a string matched to a regular expression */
typedef struct {
	Boolean	hastag;		/* true if pattern has tags */
	size_t	start, end;	/* start and end of matched portion */
	struct {					/* start and end of tagged portions */
		size_t start, end;
	} tags[MAXTAG];
} PatMatchStruct, *PatMatchType;

OSErr patcompile(PatType pat,
						uchar *str,
						Boolean ignrcase,
						size_t *where);

Boolean patsearch(PatType pat,
						uchar *str,
						size_t index,
						size_t length,
						PatMatchType match,
						OSErr *err); 

Boolean patmatch(PatType pat,
						uchar *str,
						size_t index,
						size_t length,
						PatMatchType match,
						OSErr *err);

#endif
