/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

#ifndef _H_patprivate
#define _H_patprivate

#ifndef _H_patlib
#include "patlib.h"
#endif

/*
	Various definitions for things used internally by the pattern
	functions.
*/

/* Special symbols, some of which will appear in the final pattern.
	They are all numbered above 256 so as not to conflict with any
	characters. Most of these values are also used during parsing to
	indicate the various input types. The 'P' before the names
	stands for "Pattern". The values for P_STARTTAG, P_ENDTAG, and
	P_TAGREF all have an accompanying "dummy" value which is used
	to reserve unique values for each of the 9 tags.
*/
enum {
	P_ESCAPE=257,	/* escaped sequence (used only while parsing) */
	P_LETTER,		/* letter (used only while parsing) */
	P_SGROUP,		/* start group (used only while parsing) */
	P_EGROUP,		/* end group (used only while parsing) */
	P_OR,				/* or two expressions (used only while parsing) */
	P_BOL,			/* beginning of line */
	P_EOL,			/* end of line */
	P_ANY,			/* match any character */
	P_SCCL,			/* start character class */
	P_NCCL,			/* negated character class */
	P_RANGE,			/* character range in a character class */
	P_ECCL,			/* end character class (used only while parsing) */
	P_TAGSTART,		/* start of tagged expression */
	P_TAGSTART_DUMMY = P_TAGSTART + MAXTAG,
	P_TAGEND,		/* end of tagged expression */
	P_TAGEND_DUMMY = P_TAGEND + MAXTAG,
	P_TAGREF,		/* tag reference */
	P_TAGREF_DUMMY=P_TAGREF + MAXTAG
};

/* Functions for getting the items in the pattern structure.
	Most of these "functions" are actually implemented as macros if
	assertions are turned off.
*/
Boolean		pat_ignrcase(PatType pat);
Boolean		pat_isfinal(PatType pat, PatState state);
Boolean		pat_isnull(PatType pat, PatState state);
Boolean		pat_isornull(PatType pat, PatState state);
Boolean		pat_isliteral(PatType pat, PatState state);
Boolean		pat_iscliter(PatType pat, PatState state);
Boolean		pat_isclreps(PatType pat, PatState state);
PatStateType pat_type(PatType pat, PatState state);

PatItem		pat_item(PatType pat, PatState state);
PatState		pat_next(PatType pat, PatState state);
PatState		pat_next1(PatType pat, PatState state);
PatState		pat_next2(PatType pat, PatState state);
PatReps		pat_min(PatType pat, PatState state);
PatReps		pat_max(PatType pat, PatState state);

/* Functions for setting the items in the pattern structure.
	These "functions" are actually implemented as macros if
	assertions are turned off. */
void	pat_setitem(PatType pat, PatState state, PatItem item);
void	pat_setnext(PatType pat, PatState state, PatState next);
void	pat_setnext1(PatType pat, PatState state, PatState next1);
void	pat_setnext2(PatType pat, PatState state, PatState next2);
void	pat_setmin(PatType pat, PatState state, PatReps min);
void	pat_setmax(PatType pat, PatState state, PatReps max);
void	pat_settype(PatType pat, PatState state, PatStateType type);

/* Functions for handling tags */
Boolean		pat_istagref(PatItem item);
Boolean		pat_istagstart(PatItem item);
Boolean		pat_istagend(PatItem item);
short			pat_tagref2num(PatItem p);
short			pat_tagstart2num(PatItem p);
short			pat_tagend2num(PatItem p);
PatItem		pat_num2tagref(short num);
PatItem		pat_num2tagstart(short num);
PatItem		pat_num2tagend(short num);

/* macro equivalents for most of the above functions */
#if ! ASSERTIONS
#define pat_item(p, s)		((p)->state[(s)].a.item)
#define pat_next(p, s)		((p)->state[(s)].b.next)
#define pat_next1(p, s)		((p)->state[(s)].a.next1)
#define pat_next2(p, s)		((p)->state[(s)].b.next2)
#define pat_min(p, s)		((p)->state[(s)].a.min)
#define pat_max(p, s)		((p)->state[(s)].b.max)
#define pat_ignrcase(p)		((p)->ignrcase)
#define pat_type(p, s)		((p)->state[(s)].type)
#define pat_isornull(p, s)	(pat_type(p, s) == PAT_ORNULL_STATE)
#define pat_isnull(p, s)	(pat_type(p, s) == PAT_NULL_STATE || pat_isornull(p, s))
#define pat_isliteral(p, s) (pat_type(p, s) == PAT_LITERAL_STATE)
#define pat_iscliter(p, s)	(pat_type(p, s) == PAT_CLITER_STATE)
#define pat_isclreps(p, s)	(pat_type(p, s) == PAT_CLREPS_STATE)
#endif

#if ! ASSERTIONS
#define pat_setitem(p, s, i)	((p)->state[(s)].a.item = (i))
#define pat_setnext(p, s, n)	((p)->state[(s)].b.next = (n))
#define pat_setnext1(p, s, n)	((p)->state[(s)].a.next1 = (n))
#define pat_setnext2(p, s, n)	((p)->state[(s)].b.next2 = (n))
#define pat_setmin(p, s, m)	((p)->state[(s)].a.min = (m))
#define pat_setmax(p, s, m)	((p)->state[(s)].b.max = (m))
#define pat_settype(p, s, t)	((p)->state[(s)].type = (t))
#endif

#define pat_isccl(p)				((p) == P_SCCL || (p) == P_NCCL)

#define pat_tagsize(t, i)		((t)[(i)].end - (t)[(i)].start)
#define pat_istagref(p)			(P_TAGREF <= (p) && (p) <= P_TAGREF_DUMMY)
#define pat_istagstart(p)		(P_TAGSTART <= (p) && (p) <= P_TAGSTART_DUMMY)
#define pat_istagend(p)			(P_TAGEND <= (p) && (p) <= P_TAGEND_DUMMY)

#define pat_tagref2num(p)		((p) - P_TAGREF)
#define pat_tagstart2num(p)	((p) - P_TAGSTART)
#define pat_tagend2num(p)		((p) - P_TAGEND)
#define pat_num2tagref(n)		((n) + P_TAGREF)
#define pat_num2tagstart(n)	((n) + P_TAGSTART)
#define pat_num2tagend(n)		((n) + P_TAGEND)

/* data types and functions for the stack which stores alternative
	pattern states while matching the pattern to a string.
*/

#define PAT_STACKEMPTY	(-1)	/* indicates an empty stack */

/* State stack data types */
#define PAT_MAXSTACK	MAXPAT
typedef PatState	PatStackItem;	/* an item in the stack */
typedef PatIndex	PatStackIndex;	/* index to an item in the stack */
typedef struct {
	PatStackItem	array[PAT_MAXSTACK];
	PatStackIndex	top;
} PatStack;

/* State stack functions */
void				pat_stkinit(PatStack *stk);
void				pat_stkpush(PatStack *stk, PatStackItem item);
PatStackItem	pat_stkpop(PatStack *stk);
Boolean			pat_stkempty(PatStack *stk);
Boolean			pat_stkfull(PatStack *stk);

#if ! ASSERTIONS
#define pat_stkinit(stk)			((stk)->top = 0)
#define pat_stkempty(stk)			((stk)->top == 0)
#define pat_stkfull(stk)			((stk)->top >= PAT_MAXSTACK)
#define pat_stkpush(stk, item)	((stk)->array[(stk)->top++] = (item))
#define pat_stkpop(stk)				(pat_stkempty(stk) ? PAT_STACKEMPTY : (stk)->array[--(stk)->top])
#endif

#if DEBUG
	void	printpat(PatType pat);
#endif

#endif
