/*
Copyright  1990 Ari I. Halberstadt

This file is part of a free program. For precise terms of distribution
and use please see the files in the folder Legal accompanying this
software. If you make any modifications to this file you must add the
date, your name, and a description of what you have done to the
revision history below. This notice and the copyright notice must
remain intact in all copies of this file. The files in the folder
Legal must be available to anyone receiving copies of this file.

Add initials and name here:
Initials	Name
AIH		Ari I. Halberstadt

Revision History (date format is mm-dd-yy):
Date      Name  Comments
07-11-90	 AIH   Version 0.9
*/

/*
	This file contains functions which are part of the set of
	functions used to search for strings in a HyperCard string
	or container.
	
	The function search_periodic is called whenever the periodic
	option is true. It executes periodic events, such as calling
	WaitNextEvent and spinning the cursor.
*/

#include <MyHeaders>
#include "search_private.h"

static Boolean user_canceled(SearchData *sd);
static void spin_cursor(SearchData *sd);

OSErr _search_periodic(register SearchData *sd)
{
	register size_t ticks;

	ticks = TickCount();
	if (sd->opts.period.cancel && ticks >= sd->opts.period.evt_next) {
		if (user_canceled(sd))
			return(ERR_BASIC_CANCELED);
		sd->opts.period.evt_next = ticks + sd->opts.period.evt_interval;
	}
	if (sd->opts.period.spin && ticks >= sd->opts.period.spin_next) {
		spin_cursor(sd);
		sd->opts.period.spin_next = ticks + sd->opts.period.spin_interval;
	}
	return(noErr);
}


/* check if user pressed Command-Period */
static Boolean user_canceled(SearchData *sd)
{
	Boolean canceled;
	Boolean gotEvent;
	EventRecord	event;
	
	canceled = false;
	if (HasWNE(sd->gdata))
		gotEvent = WaitNextEvent(keyDownMask + autoKeyMask, &event, 0L, NULL);
	else {
		SystemTask();
		gotEvent = GetNextEvent(keyDownMask + autoKeyMask, &event);
	}
	if (gotEvent) {
		char c = event.message & charCodeMask;
		canceled = ((event.modifiers & cmdKey) && c == '.');
	}
	return(canceled);
}

/* spin the cursor */
static void spin_cursor(SearchData *sd)
{
	SetCursor(& sd->opts.period.cursors[sd->opts.period.last_cursor++]);
	sd->opts.period.last_cursor %= NBUSY_CURS;
}
